@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,20 +24,22 @@
 #include "grt/grt_manager.h"
 #include "grts/structs.db.mgmt.h"
 #include "grts/structs.app.h"
-#include "dialogs/search_replace.h"
+//#include "dialogs/search_replace.h"
 #include <boost/weak_ptr.hpp>
+#include "base/trackable.h"
+#include "base/notifications.h"
 
-class Db_sql_editor;
+class SqlEditorForm;
 
 // Central point of management for SQLIDE (sql editor)
 namespace wb 
 {
   class WBContextUI;
   
-  class MYSQLWBBACKEND_PUBLIC_FUNC WBContextSQLIDE : public base::trackable
+  class MYSQLWBBACKEND_PUBLIC_FUNC WBContextSQLIDE : public base::trackable, base::Observer
   {
     WBContextUI *_wbui;
-    std::list<boost::weak_ptr<Db_sql_editor> > _open_editors;
+    std::list<boost::weak_ptr<SqlEditorForm> > _open_editors;
     app_ToolbarRef _toolbar;
 
     int _auto_save_interval;
@@ -53,12 +55,12 @@ namespace wb
     static void detect_auto_save_files(const std::string &autosave_dir);
     static std::map<std::string, std::string> auto_save_sessions();
     
-    boost::shared_ptr<Db_sql_editor> create_connected_editor(const db_mgmt_ConnectionRef &conn);
+    boost::shared_ptr<SqlEditorForm> create_connected_editor(const db_mgmt_ConnectionRef &conn);
 
     WBContextUI *get_wbui() { return _wbui; }
     bec::GRTManager *get_grt_manager();
     
-    Db_sql_editor* get_active_sql_editor();
+    SqlEditorForm* get_active_sql_editor();
 
     bool activate_live_object(GrtObjectRef object);
     bool create_live_object(GrtObjectRef object_type, std::string owner_name, std::string obj_name);
@@ -69,17 +71,22 @@ namespace wb
     
     bool request_quit();
     
-    void editor_will_close(Db_sql_editor*); // to be called by Db_sql_editor
+    void editor_will_close(SqlEditorForm*); // to be called by SqlEditorForm
 
   public:
     void update_plugin_arguments_pool(bec::ArgumentPool &args);
 
-    db_query_EditorRef get_grt_editor_object(Db_sql_editor *editor);
+    db_query_EditorRef get_grt_editor_object(SqlEditorForm *editor);
+    std::list<boost::weak_ptr<SqlEditorForm> >* get_open_editors() { return &_open_editors; }
   private:
-    void active_form_changed(bec::UIForm *form);
+    void call_in_editor(void (SqlEditorForm::*method)());    
+    void call_in_editor_str(void (SqlEditorForm::*method)(const std::string &arg), const std::string &arg);
+    void call_in_editor_bool(void (SqlEditorForm::*method)(bool arg), bool arg);
     
     bool auto_save_workspaces();
     void option_changed(grt::internal::OwnedDict*dict, bool, const std::string&key);
+    
+    virtual void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info);
   };
 };
                       
