@@ -28,7 +28,9 @@
 #include "home_screen.h"
 #include "base/string_utilities.h"
 
-#include "mforms/menu.h"
+#include "mforms/button.h"
+#include "mforms/imagebox.h"
+#include "mforms/menubar.h"
 
 using namespace wb;
 using namespace mforms;
@@ -597,11 +599,11 @@ HomeScreenAction WorkbenchCentral::action_from_point(int x, int y)
       grt::DictRef dict= grt::DictRef(_grt);
 
       // Compute a hot spot just below the "more" button, horizontally centered.
-      Point position((_more_starters_icon_bounds.left() + _more_starters_icon_bounds.right()) / 2,
-        _more_starters_icon_bounds.bottom() + 17);
-      client_to_screen(position);
-      dict.gset("left", (int) position.x);
-      dict.gset("top", (int) position.y);
+      int x = (int)((_more_starters_icon_bounds.left() + _more_starters_icon_bounds.right()) / 2);
+      int y = (int)(_more_starters_icon_bounds.bottom() + 17);
+      client_to_screen(x, y);
+      dict.gset("left", x);
+      dict.gset("top", y);
       _last_hit= dict;
       return ActionStarterPopup;
     }
@@ -972,8 +974,8 @@ void ActionLinkGroup::paint(cairo_t* cr, Rect bounds, bool hot, bool active)
 
 void ActionLinkGroup::mouse_click(int button, int x, int y)
 {
-    if (button == MouseButtonLeft && _arrow_bounds.contains(x,y))
-      toggle_state();
+  if (button == MouseButtonLeft && _arrow_bounds.contains(x,y))
+    toggle_state();
 }
 
 void ActionLinkGroup::mouse_double_click(int button, int x, int y)
@@ -1363,6 +1365,8 @@ void ActionLinkBox::mouse_up(int button, int x, int y)
 
 void ActionLinkBox::mouse_click(int button, int x, int y)
 {
+  _last_mouse_button = (mforms::MouseButton)button;
+  
   // Passes the call to the action link
   ActionLink* link= action_link_from_point(x, y);
 
@@ -1406,11 +1410,12 @@ void ActionLinkBox::mouse_click(int button, int x, int y)
 
 void ActionLinkBox::mouse_double_click(int button, int x, int y)
 {
+  ActionLink* link= action_link_from_point(x, y);
+      
   switch (button)
   {
     case MouseButtonLeft:
-      ActionLink* link= action_link_from_point(x, y);
-      
+    {
       // Passes the call to the action link
       if (link)
         link->mouse_double_click(button, x, y);
@@ -1419,6 +1424,7 @@ void ActionLinkBox::mouse_double_click(int button, int x, int y)
       {
         if (link == _selected_link)
         {
+          // TODO: does this make sense? Can a link (pointer) appear more than once in the list?
           for (int index= 0; index < (int) _links.size(); index++)
             if (_links[index] == _selected_link)
             {
@@ -1428,7 +1434,24 @@ void ActionLinkBox::mouse_double_click(int button, int x, int y)
         }
       }
       break;
+    }
+    case MouseButtonRight:
+    {
+      // If this right double click came from a !right + right sequence handle it as single right click.
+      if (_last_mouse_button != mforms::MouseButtonRight && (_selected_link != NULL))
+      {
+        if (link->get_type() == GroupLink)
+        {
+          if (_group_context_menu != NULL)
+            _group_context_menu->popup_at(this, x + 5, y + 5);
+        }
+        else if (_context_menu != NULL)
+          _context_menu->popup_at(this, x + 5, y + 5);
+      }
+
+    }
   }
+  _last_mouse_button = mforms::MouseButtonNone;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1615,7 +1638,7 @@ ActionList::ActionList(HomeScreen* owner, int left_spacing, int right_spacing, W
   _link_box->single_click(false);
   _link_box->set_back_color(0xeef1f5);
   _link_box->set_name("LinkBox");
-  scoped_connect(_link_box->group_toggled(),boost::bind(&ActionList::toggle_group, this, _1)); 
+  scoped_connect(_link_box->group_toggled(), boost::bind(&ActionList::toggle_group, this, _1)); 
 
 #if !defined(_WIN32) && !defined(__APPLE__)
   _link_box->set_size(-1, -1); 
@@ -2405,7 +2428,7 @@ void HomeScreen::clear_list(HomeScreenListType list)
 
 void HomeScreen::toggle_list_group(HomeScreenListType list, const std::string &group_name)
 {
-    _workspace_content->toggle_list_group(list, group_name);
+  _workspace_content->toggle_list_group(list, group_name);
 }
 
 //--------------------------------------------------------------------------------------------------
