@@ -323,7 +323,7 @@ static void sleep_2ms()
 static GThread *_main_thread= 0;
 
 GRTDispatcher::GRTDispatcher(grt::GRT *grt, bool threaded, bool is_main_dispatcher)
-  : _busy(false), _threading_disabled(!threaded), _is_main_dispatcher(is_main_dispatcher), _grt(grt), _current_task(NULL)
+  : _busy(false), _threading_disabled(!threaded), _is_main_dispatcher(is_main_dispatcher), _grt(grt), _current_task(NULL), _shut_down(false)
 {
   _worker_running= false;
   _worker_shutdown= false;
@@ -364,6 +364,7 @@ GRTDispatcher::~GRTDispatcher()
 
 void GRTDispatcher::start(boost::shared_ptr<GRTDispatcher> self)
 {
+  _shut_down = false;
   if (!_threading_disabled)
   {
     DPRINT("starting worker thread");
@@ -386,6 +387,9 @@ void GRTDispatcher::start(boost::shared_ptr<GRTDispatcher> self)
 
 void GRTDispatcher::shutdown()
 {
+  if (_shut_down)
+      return;
+  _shut_down = true;
   if (_is_main_dispatcher)
   {
     DispatcherCallbacks &cb(_default_callbacks);
