@@ -52,7 +52,7 @@ public:
   void set_fill_color2(const MySQL::Drawing::Color &color);
   void set_text_color(const MySQL::Drawing::Color &color);
   
-  boost::signals2::connection updater_connection;
+  boost::signals2::scoped_connection updater_connection;
 };
 
 
