@@ -34,11 +34,11 @@ class WBPUBLICBACKEND_PUBLIC_FUNC workbench_physical_Connection::ImplData : publ
   typedef model_Connection::ImplData super;
   
 protected:
-  boost::signals2::connection _realize_conn;
+  boost::signals2::scoped_connection _realize_conn;
 
-  boost::signals2::connection _fk_member_changed_conn;
-  boost::signals2::connection _fk_changed_conn;
-  boost::signals2::connection _table_changed_conn;
+  boost::signals2::scoped_connection _fk_member_changed_conn;
+  boost::signals2::scoped_connection _fk_changed_conn;
+  boost::signals2::scoped_connection _table_changed_conn;
 
   bool _highlighting;
 
