@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -32,26 +32,21 @@ namespace Grt {
 namespace Db {
 
 
-using namespace System;
-using namespace System::Runtime::InteropServices;
-using namespace System::Collections::Generic;
-
-
 ref class Recordset_data_storage;
 
 
-public ref class Recordset : public MySQL::Grt::VarGridModel
+public ref class RecordsetWrapper : public MySQL::Grt::VarGridModelWrapper
 {
 public:
   typedef ManagedRef<::Recordset> ^ Ref;
-  Recordset(Ref ref);
-  Recordset(IntPtr nref_ptr);
+  RecordsetWrapper(Ref ref);
+  RecordsetWrapper(IntPtr nref_ptr);
   Ref ref() { return _ref; }
   IntPtr ref_intptr() { return ~_ref; }
 private:
   Ref _ref;
 private:
-  ~Recordset();
+  ~RecordsetWrapper();
 
 public:
   MySQL::Grt::ActionList ^action_list;
@@ -61,9 +56,12 @@ public:
   String ^ caption() { return CppStringToNative(_ref->caption()); }
   void caption(String ^value) { _ref->caption(NativeToCppString(value)); }
   bool can_close() { return _ref->can_close(); }
-  bool close() { return _ref->close(); }
-  bool pinned() { return _ref->pinned(); }
-  void pinned(bool value) { _ref->pinned(value); }
+  bool close()
+  {
+    if ((void*)~_ref != NULL)
+      return _ref->close();
+    return true;
+  }
 
   String ^ status_text() { return CppStringToNative(_ref->status_text()); }
 
@@ -84,7 +82,7 @@ public:
   void sort_by(int column, int direction, bool retaining) { _ref->sort_by((::ColumnId) column, direction, retaining); }
 
   bool delete_nodes(List<NodeId^> ^nodes);
-      
+
   bool has_column_filters() { return _ref->has_column_filters(); }
   bool has_column_filter(int column) { return _ref->has_column_filter((::ColumnId) column); }
   String ^ get_column_filter_expr(int column)
@@ -102,8 +100,8 @@ public:
   void copy_rows_to_clipboard(List<int> ^indeces);
   void copy_field_to_clipboard(int row, int column) { _ref->copy_field_to_clipboard(row, column); }
 
-  void data_storages_for_export(List<String ^> ^%formats_names, List<String ^> ^%formats_desc);
-  Recordset_data_storage ^ data_storage_for_export(String ^format);
+ // void data_storages_for_export(List<String ^> ^%formats_names, List<String ^> ^%formats_desc);
+ // Recordset_data_storage ^ data_storage_for_export(String ^format);
 
 public:
   void refresh_ui_status_bar_cb(Refresh_ui_cb::ManagedDelegate ^cb);
