@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2007, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2007, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -24,12 +24,10 @@
 
 #include "sqlide/sql_editor_be.h"
 #include "GrtTemplates.h" // reference in stdafx.h is not enough because stdafx.h has subdir only effect
-#include "SigCSlot.h" // reference in stdafx.h is not enough because stdafx.h has subdir only effect
-#include "dialogs/search_replace.h"
+#include "DelegateWrapper.h" // reference in stdafx.h is not enough because stdafx.h has subdir only effect
 
 
 using namespace System;
-using namespace System::Runtime::InteropServices;
 using namespace System::Collections::Generic;
 
 
@@ -41,18 +39,25 @@ namespace Sql {
 
 ref class Recordset;
 
+public enum class KeywordList
+{
+  GeneralKeywords = Sql_editor::General_Keywords,
+  ProcedureKeywords = Sql_editor::Procedure_Keywords,
+  ClientKeywords = Sql_editor::Client_Keywords,
+  UserKeywords = Sql_editor::User_Keywords,
+};
 
-public ref class Sql_editor
+public ref class SqlEditorWrapper
 {
 public:
   typedef ManagedRef<::Sql_editor> ^ Ref;
-  Sql_editor(Ref ref);
-  Sql_editor(IntPtr nref_ptr);
+  SqlEditorWrapper(Ref ref);
+  SqlEditorWrapper(IntPtr nref_ptr);
   Ref ref() { return _ref; }
 private:
   Ref _ref;
 private:
-  ~Sql_editor();
+  ~SqlEditorWrapper();
 
 public:
   void reset_fe_callbacks();
@@ -142,50 +147,9 @@ private:
   ChangeCursorPosDelegate^ _change_cursor_pos_delegate;
 
 public:
-  // TODO: move this to a generic base wrapper.
-  enum class SearchFlags
-  {
-    SearchNone= mforms::SearchNone,
-    SearchMatchCase= mforms::SearchMatchCase,
-    SearchMatchWholeWord=mforms::SearchMatchWholeWord,
-    SearchUseRegularExpression=mforms::SearchUseRegularExpression,
-    SearchDoReplace=mforms::SearchDoReplace,
-    SearchAll=mforms::SearchAll,
-    SearchPrevious=mforms::SearchPrevious,
-  }; 
-
-  typedef DelegateSlot3<
-    bool, bool,
-    std::string, String^,
-    std::string, String^,
-    int, int> DoSearchDelegate;
-
-  mforms::SearchReplace *_search_replace_dialog;
-
-  void set_do_search_delegate(DoSearchDelegate::ManagedDelegate ^cb)
-  {
-    _do_search_delegate= gcnew DoSearchDelegate(cb);
-  }
-
-  void show_find_replace_dialog()
-  {
-    // We cannot re-use the dialog, as it automatically gets disposed on Windows when closed.
-    // Needs investigation if that can be prevented.
-    // However, we *have* to delete the mforms part of that dialog.
-    delete _search_replace_dialog;
-    _search_replace_dialog= new mforms::SearchReplace();
-
-    _search_replace_dialog->set_callback(_do_search_delegate->get_slot());
-
-    _search_replace_dialog->show(false, mforms::SearchNone, true);
-  }
-
   List<MySQL::Grt::MenuItem^>^ get_context_menu();
   void activate_context_menu_item(String^ action);
 
-private:
-  DoSearchDelegate^ _do_search_delegate;
-
 public:
   void check_sql(bool sync) { _ref->check_sql(sync); }
 
@@ -222,6 +186,9 @@ private:
 
 public:
   void get_sql_statement_border_by_line_pos(int lineno, int line_pos, int %begin_lineno, int %begin_line_pos, int %end_lineno, int %end_line_pos);
+
+  String^ get_keywords(KeywordList list);
+  String^ get_function_names();
 };
 
 
