@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 #
 
 """
@@ -25,7 +25,7 @@ import csv
 import re
 import sys
 from itertools import imap
-from mysql.utilities.exception import MySQLUtilError
+from mysql.utilities.exception import UtilError
 
 # List of database objects for enumeration
 _DATA_DECORATE = "DATA FOR TABLE"
@@ -356,7 +356,10 @@ def _build_create_table(db_name, tbl_name, engine, columns, col_ref={}):
             constraint_str += "REFERENCES `%s` (`%s`)" % \
                               (constraint[1], constraint[3])
             create_str += ",\n" + constraint_str
-    create_str += "\n) ENGINE=%s;" % engine
+    create_str += "\n)"
+    if engine is not None and len(engine) > 0:
+        create_str += " ENGINE=%s" % engine
+    create_str += ";"
     return create_str
 
 
@@ -396,6 +399,7 @@ def _build_create_objects(obj_type, db, definitions):
     col_list = []
     stop = len(definitions)
     col_ref = {}
+    engine = None
     # Now the tricky part.
     for i in range(0,stop):
         if skip_header:
@@ -403,6 +407,9 @@ def _build_create_objects(obj_type, db, definitions):
             col_ref = _build_column_ref(definitions[i])
             continue
         defn = definitions[i]
+        # Read engine from first row and save old value.
+        old_engine = engine
+        engine = defn[col_ref.get("ENGINE",2)]
         create_str = ""
         if obj_type == "TABLE":
             if (obj_db == "" and obj_name == ""):
@@ -412,11 +419,8 @@ def _build_create_objects(obj_type, db, definitions):
                 obj_name == defn[col_ref.get("TABLE_NAME",1)]):
                 col_list.append(defn)
             else:
-                # this is a new one, so make the CREATE statement.
-                # we should capture table data from first row like
-                # engine, etc. then add cols to col_list.
                 create_str = _build_create_table(obj_db, obj_name,
-                                                 defn[col_ref.get("ENGINE",2)],
+                                                 old_engine,
                                                  col_list, col_ref)
                 create_strings.append(create_str)
                 obj_db = defn[col_ref.get("TABLE_SCHEMA",0)]
@@ -426,7 +430,7 @@ def _build_create_objects(obj_type, db, definitions):
             # check for end.
             if i+1 == stop:
                 create_str = _build_create_table(obj_db, obj_name,
-                                                 defn[col_ref.get("ENGINE",2)],
+                                                 engine,
                                                  col_list, col_ref)
                 create_strings.append(create_str)
         elif obj_type == "VIEW":
@@ -484,7 +488,7 @@ def _build_create_objects(obj_type, db, definitions):
             try:
                 user, priv, db, tbl = defn[0:4]
             except:
-                raise MySQLUtilError("Object data invalid: %s : %s" % \
+                raise UtilError("Object data invalid: %s : %s" % \
                                      (obj_type, defn))
             if not tbl:
                 tbl = "*"
@@ -493,7 +497,7 @@ def _build_create_objects(obj_type, db, definitions):
             create_str = "GRANT %s ON %s.%s TO %s" % (priv, db, tbl, user)
             create_strings.append(create_str)
         else:
-            raise MySQLUtilError("Unknown object type discovered: %s", obj_type)
+            raise UtilError("Unknown object type discovered: %s", obj_type)
     return create_strings
 
 
@@ -657,16 +661,27 @@ def _exec_statements(statements, destination, format, options, dryrun=False):
 
     Returns (bool) - True if all execute, raises error if one fails
     """
+    new_engine = options.get("new_engine", None)
+    def_engine = options.get("def_engine", None)
+    quiet = options.get("quiet", False)
     for statement in statements:
-        if dryrun:
-            print statement
-        else:
-            try:
-                if format != "SQL" or not _skip_sql(statement, options):
-                    destination.exec_query(statement)
-            except MySQLUtilError, e:
-                raise MySQLUtilError("Invalid statement:\n%s" %
-                                     statement)
+        if (new_engine is not None or def_engine is not None) and \
+           statement.upper()[0:12] == "CREATE TABLE":
+            i = statement.find(' ', 13)
+            tbl_name = statement[13:i]
+            statement = destination.substitute_engine(tbl_name, statement,
+                                                      new_engine, def_engine,
+                                                      quiet)
+        try:
+            if dryrun:
+                print statement
+            elif format != "SQL" or not _skip_sql(statement, options):
+                destination.exec_query(statement)
+        # Here we capture any exception and raise UtilError to communicate to
+        # the script/user. Since all util errors (exceptions) derive from
+        # Exception, this is safe.
+        except Exception, e:
+            raise UtilError("Invalid statement:\n%s" % statement)
     return True
 
 
@@ -713,6 +728,7 @@ def import_file(dest_val, file_name, options):
     """
 
     from mysql.utilities.common.database import Database
+    from mysql.utilities.common.options import check_engine_options
     from mysql.utilities.common.table import Table
     from mysql.utilities.common.server import connect_servers
 
@@ -741,7 +757,7 @@ def import_file(dest_val, file_name, options):
             fix_cols.append((tbl.tbl_name, columns))
             col_meta = _get_column_metadata(tbl, fix_cols)
         if col_meta is None:
-            raise MySQLUtilError("Cannot build bulk insert statements without "
+            raise UtilError("Cannot build bulk insert statements without "
                                  "the table definition.")
         ins_strs = tbl.make_bulk_insert(table_rows, tbl.db_name, col_meta)
         if len(ins_strs[0]) > 0:
@@ -761,10 +777,20 @@ def import_file(dest_val, file_name, options):
     skip_blobs = options.get("skip_blobs", False)
 
     # Attempt to connect to the destination server
-    servers = connect_servers(dest_val, None, quiet, "5.1.30")
+    conn_options = {
+        'quiet'     : quiet,
+        'version'   : "5.1.30",
+    }
+    servers = connect_servers(dest_val, None, conn_options)
 
     destination = servers[0]
 
+    # Check storage engines
+    check_engine_options(destination,
+                         options.get("new_engine", None),
+                         options.get("def_engine", None),
+                         False, options.get("quiet", False))
+
     if not quiet:
         if import_type == "BOTH":
             str = "definitions and data"
@@ -809,17 +835,18 @@ def import_file(dest_val, file_name, options):
         # check user permissions on source for all databases
         if db_name is not None:
             dest_db = Database(destination, db_name)
-            dest_db.check_write_access(dest_val["user"], dest_val["host"],
-                                       options.get("skip_views", False),
-                                       options.get("skip_procs", False),
-                                       options.get("skip_funcs", False),
-                                       options.get("skip_grants", False))
+
+            # Make a dictionary of the options
+            access_options = options.copy()
+
+            dest_db.check_write_access(dest_val['user'], dest_val['host'],
+                                       access_options)
 
         # Now check to see if we want definitions, data, or both:
         if row[0] == "SQL" or row[0] in _DEFINITION_LIST:
             if format != "SQL" and len(row[1]) == 1:
-                raise MySQLUtilError("Cannot read an import file "
-                                      "generated with --display=NAMES")
+                raise UtilError("Cannot read an import file generated with "
+                                "--display=NAMES")
 
             if import_type == "DEFINITIONS" or import_type == "BOTH":
                 if format == "SQL":
@@ -867,6 +894,12 @@ def import_file(dest_val, file_name, options):
                                 str = _build_insert_data(columns, tbl_name,
                                                          row[1])
                                 statements.append(str)
+
+    # Process remaining definitions                                 
+    if len(definitions) > 0:
+        _process_definitions(statements, table_col_list, db_name)
+        definitions = []
+
     # Process remaining data rows
     if len(table_rows) > 0:
         _process_data(tbl_name, statements, columns,
