@@ -13,7 +13,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 #
 
 """
@@ -29,7 +29,7 @@ import shutil
 
 def clone_server(conn_val, new_data, new_port, new_id, rootpass,
                  mysqld_options=None, verbose=False, quiet=False):
-    """ Clone an existing server
+    """Clone an existing server
 
     This method creates a new instance of a running server using a datadir
     set to the new_data parametr, with a port set to new_port, server_id
@@ -58,11 +58,15 @@ def clone_server(conn_val, new_data, new_port, new_id, rootpass,
     """
 
     from mysql.utilities.common.server import Server
-    from mysql.utilities.exception import MySQLUtilError
+    from mysql.utilities.exception import UtilError
     from mysql.utilities.common.tools import get_tool_path
 
     # Try to connect to the MySQL database server.
-    server1 = Server(conn_val, "source")
+    server1_options = {
+        'conn_info' : conn_val,
+        'role'      : "source",
+    }
+    server1 = Server(server1_options)
     server1.connect()
 
     if not quiet:
@@ -79,7 +83,7 @@ def clone_server(conn_val, new_data, new_port, new_id, rootpass,
         try:
             res = os.mkdir(new_data)
         except:
-            raise MySQLUtilError("Unable to create directory '%s'" % new_data)
+            raise UtilError("Unable to create directory '%s'" % new_data)
 
     basedir = ""
     # Get basedir
@@ -89,7 +93,7 @@ def clone_server(conn_val, new_data, new_port, new_id, rootpass,
     if rows:
         basedir = rows[0][1]
     else:
-        raise MySQLUtilError("Unable to determine basedir of running server.")
+        raise UtilError("Unable to determine basedir of running server.")
 
     if not quiet:
         print "# Locating mysql tools..."
@@ -182,7 +186,11 @@ def clone_server(conn_val, new_data, new_port, new_id, rootpass,
         "port"   : port_int,
         "unix_socket" : new_sock
     }
-    server2 = Server(conn, "clone")
+    server2_options = {
+        'conn_info' : conn,
+        'role'      : "clone",
+    }
+    server2 = Server(server2_options)
 
     stop = 10 # stop after 10 attempts
     i = 0
@@ -199,7 +207,7 @@ def clone_server(conn_val, new_data, new_port, new_id, rootpass,
                 print "# trying again..."
 
     if i == stop:
-        raise MySQLUtilError("Unable to communicate with new instance.")
+        raise UtilError("Unable to communicate with new instance.")
     elif not quiet:
         print "# Success!"
 
