@@ -12,7 +12,7 @@
 #
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307 USA
+# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA
 #
 
 """
@@ -23,7 +23,7 @@ import multiprocessing
 import re
 import sys
 import mysql.connector
-from mysql.utilities.exception import MySQLUtilError
+from mysql.utilities.exception import UtilError
 
 # Constants
 _MAXPACKET_SIZE = 1024 * 1024
@@ -250,7 +250,7 @@ class Index(object):
         return (self.db, self.table, self.name, self.type, cols)
 
 
-class Table:
+class Table(object):
     """
     The Table class encapsulates a table for a given database. The class
     has the following capabilities:
@@ -263,22 +263,26 @@ class Table:
         - Copy table data
     """
 
-    def __init__(self, server1, name, verbose=False, get_cols=False):
+    def __init__(self, server1, name, options={}):
         """Constructor
 
         server[in]         A Server object
         name[in]           Name of table in the form (db.table)
-        verbose[in]        print extra data during operations (optional)
-                           (default is False)
-        get_cols[in]       If True, get the column metadata on construction
-                           (default is False)
+        options[in]        options for class: verbose, quiet, get_cols,
+            quiet     If True, do not print information messages    
+            verbose   print extra data during operations (optional)
+                      (default is False)
+            get_cols  If True, get the column metadata on construction
+                      (default is False)
         """
 
-        self.verbose = verbose
+        self.verbose = options.get('verbose', False)
+        self.quiet = options.get('quiet', False)
         self.server = server1
         self.table = name
         self.db_name, self.tbl_name = _parse_object_name(name)
         self.obj_type = "TABLE"
+        self.pri_idx = None
 
         # We store each type of index in a separate list to make it easier
         # to manipulate
@@ -286,7 +290,7 @@ class Table:
         self.hash_indexes = []
         self.rtree_indexes = []
         self.fulltext_indexes = []
-        if get_cols:
+        if options.get('get_cols', False):
             self.col_metadata = self.get_column_metadata()
         else:
             self.col_metadata = None
@@ -303,6 +307,9 @@ class Table:
             self.max_packet_size = _MAXPACKET_SIZE
 
         self._insert = "INSERT INTO %s.%s VALUES "
+        self.query_options = {  # Used for skipping fetch of rows
+            'fetch' : False
+        }
 
 
     def exists(self, tbl_name=None):
@@ -392,7 +399,7 @@ class Table:
 
 
     def _build_update_blob(self, row, new_db, name, special_cols, blob_col):
-        """ Build an UPDATE statement to update blob fields.
+        """Build an UPDATE statement to update blob fields.
 
         row[in]            a row to process
         new_db[in]         new database name
@@ -538,7 +545,7 @@ class Table:
         num_rows = 0
         try:
             res = self.server.exec_query("USE %s" % self.db_name,
-                                         (), False, False)
+                                         self.query_options)
         except Exception, e:
             pass
         res = self.server.exec_query("SHOW TABLE STATUS LIKE '%s'" % \
@@ -587,7 +594,11 @@ class Table:
             self.dest_vals = self.get_dest_values(destination)
 
         # Spawn a new connection
-        dest = Server(self.dest_vals, "thread")
+        server_options = {
+            'conn_info' : self.dest_vals,
+            'role'      : "thread",
+        }
+        dest = Server(server_options)
         dest.connect()
 
         # First, turn off foreign keys if turned on
@@ -603,9 +614,9 @@ class Table:
         # Insert the data first
         for data_insert in insert_data:
             try:
-                res = dest.exec_query(data_insert, (), False, False)
-            except MySQLUtilError, e:
-                raise MySQLUtilError("Problem inserting data. "
+                res = dest.exec_query(data_insert, self.query_options)
+            except UtilError, e:
+                raise UtilError("Problem inserting data. "
                                      "Error = %s" % e.errmsg)
 
         # Now insert the blob data if there is any
@@ -613,9 +624,9 @@ class Table:
             try:
                 # Must convert blob data to a raw string for cursor to handle.
                 res = dest.exec_query(blob_insert[0] % "%r" % blob_insert[1],
-                                      (), False, False)
-            except MySQLUtilError, e:
-                raise MySQLUtilError("Problem updating blob field. "
+                                      self.query_options)
+            except UtilError, e:
+                raise UtilError("Problem updating blob field. "
                                      "Error = %s" % e.errmsg)
 
         # Now, turn on foreign keys if they were on at the start
@@ -715,7 +726,7 @@ class Table:
 
         # Execute query to get all of the data
         cur = self.server.exec_query("SELECT * FROM %s" % self.table,
-                                     (), False, False, True)
+                                     self.query_options)
 
         while True:
             rows = None
@@ -842,11 +853,31 @@ class Table:
 
 
     def _get_index_list(self):
-        """ Get the list of indexes for a table.
+        """Get the list of indexes for a table.
         Returns list containing indexes.
         """
         rows = self.get_tbl_indexes()
         return rows
+    
+    
+    def get_primary_index(self):
+        """Retrieve the primary index columns for this table.
+        """
+        pri_idx = []
+        
+        rows = self.server.exec_query("EXPLAIN " + self.table)
+
+        # Return False if no indexes found.
+        if not rows:
+            return pri_idx
+        
+        for row in rows:
+            if row[3] == 'PRI':
+                pri_idx.append(row)
+
+        self.pri_idx = pri_idx
+        
+        return pri_idx            
 
 
     def get_indexes(self):
@@ -887,11 +918,10 @@ class Table:
         return True
 
 
-    def check_indexes(self, show_drops=False, quiet=False):
+    def check_indexes(self, show_drops=False):
         """Check for duplicate or redundant indexes and display all matches
 
         show_drops[in]     (optional) If True the DROP statements are printed
-        quiet[in]         (optional) If True, do not print info messages
 
         Note: You must call get_indexes() prior to calling this method. If
         get_indexes() is not called, no duplicates will be found.
@@ -933,7 +963,7 @@ class Table:
                     print "%s;" % (index.get_drop_statement())
                 print "#"
         else:
-            if not quiet:
+            if not self.quiet:
                 print "# Table %s has no duplicate indexes." % (self.table)
 
 
@@ -984,16 +1014,19 @@ class Table:
 
         from mysql.utilities.common.format import print_list
 
+        query_options = {
+            'params' : (self.db_name, self.tbl_name,)
+        }
         rows = []
         type = "best"
         if not best:
             type = "worst"
         if best:
             rows= self.server.exec_query(_QUERY + "DESC LIMIT %s" % limit,
-                                         (self.db_name, self.tbl_name,))
+                                         query_options)
         else:
             rows= self.server.exec_query(_QUERY + "LIMIT %s" % limit,
-                                         (self.db_name, self.tbl_name,))
+                                         query_options)
         if rows:
             print "#"
             print "# Showing the top 5 %s performing indexes from %s:\n#" % \
@@ -1047,7 +1080,7 @@ class Table:
 
 
     def has_primary_key(self):
-        """ Check to see if there is a primary key.
+        """Check to see if there is a primary key.
         Returns bool - True - a primary key was found,
                        False - no primary key.
         """
