@@ -4,7 +4,7 @@
  *
  * Created by Mike Lischke.
  *
- * Copyright 2009 Sun Microsystems, Inc. All rights reserved.
+ * Copyright 2009, 2011 Sun Microsystems, Inc. All rights reserved.
  * This file is dual licensed under LGPL v2.1 and the Scintilla license (http://www.scintilla.org/License.txt).
  */
 
@@ -115,7 +115,6 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
   [super resetCursorRects];
   
   // We only have one cursor rect: our bounds.
-  NSRect bounds = [self bounds];
   [self addCursorRect: [self bounds] cursor: mCurrentCursor];
   [mCurrentCursor setOnMouseEntered: YES];
 }
@@ -160,6 +159,7 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
  */
 - (BOOL) acceptsFirstMouse: (NSEvent *) theEvent
 {
+#pragma unused(theEvent)
   return YES;
 }
 
@@ -242,7 +242,14 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 {
   // Remove any previously marked text first.
   [self removeMarkedText];
-  mOwner.backend->InsertText((NSString*) aString);
+  NSString* newText = @"";
+  if ([aString isKindOfClass:[NSString class]])
+    newText = (NSString*) aString;
+  else
+    if ([aString isKindOfClass:[NSAttributedString class]])
+      newText = (NSString*) [aString string];
+  
+  mOwner.backend->InsertText(newText);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -256,8 +263,8 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 
 - (NSRange) selectedRange
 {
-  int begin = [mOwner getGeneralProperty: SCI_GETSELECTIONSTART parameter: 0];
-  int end = [mOwner getGeneralProperty: SCI_GETSELECTIONEND parameter: 0];
+  long begin = [mOwner getGeneralProperty: SCI_GETSELECTIONSTART parameter: 0];
+  long end = [mOwner getGeneralProperty: SCI_GETSELECTIONEND parameter: 0];
   return NSMakeRange(begin, end - begin);
 }
 
@@ -273,18 +280,21 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
  */
 - (void) setMarkedText: (id) aString selectedRange: (NSRange) range
 {
-  // Since we did not return any valid attribute for marked text (see validAttributesForMarkedText)
-  // we can safely assume the passed in text is an NSString instance.
-  NSString* newText = (NSString*) aString;
-  int currentPosition = [mOwner getGeneralProperty: SCI_GETCURRENTPOS parameter: 0];
+  NSString* newText = @"";
+  if ([aString isKindOfClass:[NSString class]])
+    newText = (NSString*) aString;
+  else
+    if ([aString isKindOfClass:[NSAttributedString class]])
+      newText = (NSString*) [aString string];
+  
+  long currentPosition = [mOwner getGeneralProperty: SCI_GETCURRENTPOS parameter: 0];
 
   // Replace marked text if there is one.
   if (mMarkedTextRange.length > 0)
   {
-    // We have already marked text. Replace that.
     [mOwner setGeneralProperty: SCI_SETSELECTIONSTART
                          value: mMarkedTextRange.location];
-    [mOwner setGeneralProperty: SCI_SETSELECTIONEND
+    [mOwner setGeneralProperty: SCI_SETSELECTIONEND 
                          value: mMarkedTextRange.location + mMarkedTextRange.length];
     currentPosition = mMarkedTextRange.location;
   }
@@ -295,12 +305,12 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
   // Note: Scintilla internally works almost always with bytes instead chars, so we need to take
   //       this into account when determining selection ranges and such.
   std::string raw_text = [newText UTF8String];
-  mOwner.backend->InsertText(newText);
+  int lengthInserted = mOwner.backend->InsertText(newText);
 
   mMarkedTextRange.location = currentPosition;
-  mMarkedTextRange.length = raw_text.size();
+  mMarkedTextRange.length = lengthInserted;
     
-  if (mMarkedTextRange.length > 0)
+  if (lengthInserted > 0)
   {
     // Mark the just inserted text. Keep the marked range for later reset.
     [mOwner setGeneralProperty: SCI_SETINDICATORCURRENT value: INPUT_INDICATOR];
@@ -311,7 +321,7 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
   else
     // Re-enable undo action collection if composition ended (indicated by an empty mark string).
     [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: range.length == 0];
- 
+  
   // Select the part which is indicated in the given range. It does not scroll the caret into view.
   if (range.length > 0)
   {
@@ -356,7 +366,7 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
     mOwner.backend->InsertText(@"");
     mMarkedTextRange = NSMakeRange(NSNotFound, 0);
 
-    // Reenable undo action collection, after we are done with text composition.
+    // Reenable undo action collection, after we are done with text composition.    
     [mOwner setGeneralProperty: SCI_SETUNDOCOLLECTION value: 1];
   }
 }
@@ -379,7 +389,8 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
  */
 - (void) keyDown: (NSEvent *) theEvent
 {
-  mOwner.backend->KeyboardInput(theEvent);
+  if (mMarkedTextRange.length == 0)
+    mOwner.backend->KeyboardInput(theEvent);
   NSArray* events = [NSArray arrayWithObject: theEvent];
   [self interpretKeyEvents: events];
 }
@@ -489,6 +500,7 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 
 - (BOOL) prepareForDragOperation: (id <NSDraggingInfo>) sender
 {
+#pragma unused(sender)
   return YES;
 }
 
@@ -506,7 +518,19 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
  */
 - (NSDragOperation) draggingSourceOperationMaskForLocal: (BOOL) flag
 {
-  return NSDragOperationCopy | NSDragOperationMove;
+  return NSDragOperationCopy | NSDragOperationMove | NSDragOperationDelete;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Finished a drag: may need to delete selection.
+ */
+
+- (void) draggedImage: (NSImage *) image endedAt: (NSPoint) screenPoint operation: (NSDragOperation) operation
+{
+  if (operation == NSDragOperationDelete)
+    mOwner.backend->WndProc(SCI_CLEAR, 0, 0);
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -526,36 +550,43 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 
 - (void) selectAll: (id) sender
 {
+#pragma unused(sender)
   mOwner.backend->SelectAll();
 }
 
 - (void) deleteBackward: (id) sender
 {
+#pragma unused(sender)
   mOwner.backend->DeleteBackward();
 }
 
 - (void) cut: (id) sender
 {
+#pragma unused(sender)
   mOwner.backend->Cut();
 }
 
 - (void) copy: (id) sender
 {
+#pragma unused(sender)
   mOwner.backend->Copy();
 }
 
 - (void) paste: (id) sender
 {
+#pragma unused(sender)
   mOwner.backend->Paste();
 }
 
 - (void) undo: (id) sender
 {
+#pragma unused(sender)
   mOwner.backend->Undo();
 }
 
 - (void) redo: (id) sender
 {
+#pragma unused(sender)
   mOwner.backend->Redo();
 }
 
@@ -644,7 +675,7 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
 //--------------------------------------------------------------------------------------------------
 
 /**
- * Called by a connected compontent (usually the info bar) if something changed there.
+ * Called by a connected component (usually the info bar) if something changed there.
  *
  * @param type The type of the notification.
  * @param message Carries the new status message if the type is a status message change.
@@ -659,11 +690,13 @@ NSString *SCIMarginClickNotification = @"SCIMarginClick";
     case IBNZoomChanged:
     {
       // Compute point increase/decrease based on default font size.
-      int fontSize = [self getGeneralProperty: SCI_STYLEGETSIZE parameter: STYLE_DEFAULT];
+      long fontSize = [self getGeneralProperty: SCI_STYLEGETSIZE parameter: STYLE_DEFAULT];
       int zoom = (int) (fontSize * (value - 1));
       [self setGeneralProperty: SCI_SETZOOM value: zoom];
       break;
     }
+    default:
+      break;
   };
 }
 
@@ -705,43 +738,49 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
     {
       // Parent notification. Details are passed as SCNotification structure.
       SCNotification* scn = reinterpret_cast<SCNotification*>(lParam);
-      editor = reinterpret_cast<InnerView*>(scn->nmhdr.idFrom).owner;
+      ScintillaCocoa *psc = reinterpret_cast<ScintillaCocoa*>(scn->nmhdr.hwndFrom);
+      editor = reinterpret_cast<InnerView*>(psc->ContentView()).owner;
       switch (scn->nmhdr.code)
       {
         case SCN_MARGINCLICK:
         {
-          int line = [editor getGeneralProperty: SCI_LINEFROMPOSITION parameter: scn->position];
+          long line = [editor getGeneralProperty: SCI_LINEFROMPOSITION parameter: scn->position];
           if (scn->margin == 2)
           {
             // Click on the folder margin. Toggle the current line if possible.
             [editor setGeneralProperty: SCI_TOGGLEFOLD value: line];
           }
           
+          // Notification to handle special events in editors
+          // e.g. like setting breakpoints or do other actions when the gutter is clicked.
           NSMutableDictionary* info = [NSMutableDictionary dictionary];
           [info setValue: [NSNumber numberWithInt: line] forKey: @"line"];
           [info setValue: [NSNumber numberWithInt: scn->modifiers] forKey: @"modifiers"];
           [info setValue: [NSNumber numberWithInt: scn->margin] forKey: @"margin"];
           [editor sendNotification: SCIMarginClickNotification withUserInfo: info];
           break;
-          };
+        };
         case SCN_MODIFIED:
         {
-          int line = [editor getGeneralProperty: SCI_LINEFROMPOSITION parameter: scn->position];
-          NSMutableDictionary* info = [NSMutableDictionary dictionary];
-          [info setValue: [NSNumber numberWithInt: line] forKey: @"line"];
-          [info setValue: [NSNumber numberWithInt: scn->linesAdded] forKey: @"linesAdded"];
-
           // Decide depending on the modification type what to do.
           // There can be more than one modification carried by one notification.
           if (scn->modificationType & (SC_MOD_INSERTTEXT | SC_MOD_DELETETEXT))
+          {
+            // Notification used to ease parsing of text changes (e.g. for marking errors).
+            long line = [editor getGeneralProperty: SCI_LINEFROMPOSITION parameter: scn->position];
+            NSMutableDictionary* info = [NSMutableDictionary dictionary];
+            [info setValue: [NSNumber numberWithInt: line] forKey: @"line"];
+            [info setValue: [NSNumber numberWithInt: scn->linesAdded] forKey: @"linesAdded"];
+
             [editor sendNotification: NSTextDidChangeNotification withUserInfo: info];
+          }
           break;
         }
         case SCN_ZOOM:
         {
           // A zoom change happend. Notify info bar if there is one.
           float zoom = [editor getGeneralProperty: SCI_GETZOOM parameter: 0];
-          int fontSize = [editor getGeneralProperty: SCI_STYLEGETSIZE parameter: STYLE_DEFAULT];
+          long fontSize = [editor getGeneralProperty: SCI_STYLEGETSIZE parameter: STYLE_DEFAULT];
           float factor = (zoom / fontSize) + 1;
           [editor->mInfoBar notify: IBNZoomChanged message: nil location: NSZeroPoint value: factor];
           break;
@@ -751,19 +790,11 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
           // Triggered whenever changes in the UI state need to be reflected.
           // These can be: caret changes, selection changes etc.
           NSPoint caretPosition = editor->mBackend->GetCaretPosition();
-          
           [editor->mInfoBar notify: IBNCaretChanged message: nil location: caretPosition value: 0];
+          [editor sendNotification: SCIUpdateUINotification];
           [editor sendNotification: NSTextViewDidChangeSelectionNotification];
-
-          // For the UI notification we also need the caret location, but as global position,
-          // not in row/column format.
-          int position = [editor getGeneralProperty: SCI_GETCURRENTPOS];
-          NSMutableDictionary* info = [NSMutableDictionary dictionary];
-          [info setValue: [NSNumber numberWithInt: position] forKey: @"currentPosition"];
-          [editor sendNotification: SCIUpdateUINotification withUserInfo: info];
-
           break;
-        }
+      }
       }
       break;
     }
@@ -827,6 +858,17 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
     [self setGeneralProperty: SCI_INDICSETUNDER parameter: INPUT_INDICATOR value: 1];
     [self setGeneralProperty: SCI_INDICSETSTYLE parameter: INPUT_INDICATOR value: INDIC_ROUNDBOX];
     [self setGeneralProperty: SCI_INDICSETALPHA parameter: INPUT_INDICATOR value: 100];
+      
+    NSNotificationCenter *center = [NSNotificationCenter defaultCenter];
+    [center addObserver:self
+               selector:@selector(applicationDidResignActive:)
+                   name:NSApplicationDidResignActiveNotification
+                 object:nil];
+      
+    [center addObserver:self
+               selector:@selector(applicationDidBecomeActive:)
+                   name:NSApplicationDidBecomeActiveNotification
+                 object:nil];
   }
   return self;
 }
@@ -842,6 +884,22 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 
 //--------------------------------------------------------------------------------------------------
 
+- (void) applicationDidResignActive: (NSNotification *) note
+{
+#pragma unused(note)
+  mBackend->ActiveStateChanged(false);
+}
+
+//--------------------------------------------------------------------------------------------------
+
+- (void) applicationDidBecomeActive: (NSNotification *) note
+{
+#pragma unused(note)
+  mBackend->ActiveStateChanged(true);
+}
+
+//--------------------------------------------------------------------------------------------------
+
 - (void) viewDidMoveToWindow
 {
   [super viewDidMoveToWindow];
@@ -1013,7 +1071,8 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 - (BOOL) setHorizontalScrollRange: (int) range page: (int) page
 {
   BOOL result = NO;
-  BOOL hideScroller = page >= range;
+  BOOL hideScroller = (page >= range) || 
+    (mBackend->WndProc(SCI_GETWRAPMODE, 0, 0) != SC_WRAP_NONE);
   
   if ([mHorizontalScroller isHidden] != hideScroller)
   {
@@ -1084,7 +1143,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
   NSString *result = @"";
   
   char *buffer(0);
-  const int length = mBackend->WndProc(SCI_GETSELTEXT, 0, 0);
+  const long length = mBackend->WndProc(SCI_GETSELTEXT, 0, 0);
   if (length > 0)
   {
     buffer = new char[length + 1];
@@ -1117,7 +1176,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
   NSString *result = @"";
   
   char *buffer(0);
-  const int length = mBackend->WndProc(SCI_GETLENGTH, 0, 0);
+  const long length = mBackend->WndProc(SCI_GETLENGTH, 0, 0);
   if (length > 0)
   {
     buffer = new char[length + 1];
@@ -1283,23 +1342,6 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 
 //--------------------------------------------------------------------------------------------------
 
-/**
- * Specialized property setter for colors without additional parameter.
- */
-- (void) setColorProperty: (int) property value: (NSColor*) value
-{
-  if ([value colorSpaceName] != NSDeviceRGBColorSpace)
-    value = [value colorUsingColorSpaceName: NSDeviceRGBColorSpace];
-  long red = [value redComponent] * 255;
-  long green = [value greenComponent] * 255;
-  long blue = [value blueComponent] * 255;
-  
-  long color = (blue << 16) + (green << 8) + red;
-  mBackend->WndProc(property, color, 0);
-}
-
-//--------------------------------------------------------------------------------------------------
-
 /**
  * Another color property setting, which allows to specify the color as string like in HTML
  * documents (i.e. with leading # and either 3 hex digits or 6).
@@ -1373,7 +1415,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
  */
 - (NSColor*) getColorProperty: (int) property parameter: (long) parameter
 {
-  int color = mBackend->WndProc(property, parameter, 0);
+  long color = mBackend->WndProc(property, parameter, 0);
   float red = (color & 0xFF) / 255.0;
   float green = ((color >> 8) & 0xFF) / 255.0;
   float blue = ((color >> 16) & 0xFF) / 255.0;
@@ -1448,6 +1490,17 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
   return [NSString stringWithUTF8String: result];
 }
 
+//--------------------------------------------------------------------------------------------------
+
+/**
+ * Sets the notification callback
+ */
+- (void) registerNotifyCallback: (intptr_t) windowid value: (Scintilla::SciNotifyFunc) callback
+{
+  mBackend->RegisterNotifyCallback(windowid, callback);
+}
+
+
 //--------------------------------------------------------------------------------------------------
 
 /**
@@ -1594,22 +1647,23 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
 /**
  * Searches the given text and replaces
  *
- * @result YES if something was found, NO otherwise.
+ * @result Number of entries replaced, 0 if none.
  */
-- (BOOL) findAndReplaceText: (NSString*) searchText
-                     byText: (NSString*) newText
-                  matchCase: (BOOL) matchCase
-                  wholeWord: (BOOL) wholeWord
-                      doAll: (BOOL) doAll
+- (int) findAndReplaceText: (NSString*) searchText
+                    byText: (NSString*) newText
+                 matchCase: (BOOL) matchCase
+                 wholeWord: (BOOL) wholeWord
+                     doAll: (BOOL) doAll
 {
   // The current position is where we start searching for single occurences. Otherwise we start at
   // the beginning of the document.
   int startPosition;
   if (doAll)
-    startPosition = [self getGeneralProperty: SCI_GETCURRENTPOS parameter: 0];
+    startPosition = 0; // Start at the beginning of the text if we replace all occurrences.
   else
-    startPosition = 0;
-  int endPosition = [self getGeneralProperty: SCI_GETTEXTLENGTH parameter: 0];
+    // For a signle replacement we start at the current caret position.
+    startPosition = [self getGeneralProperty: SCI_GETCURRENTPOS];
+  int endPosition = [self getGeneralProperty: SCI_GETTEXTLENGTH];
 
   int searchFlags= 0;
   if (matchCase)
@@ -1626,7 +1680,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
   int targetLength = strlen(replacement);  // Length in bytes.
   sptr_t result;
   
-  BOOL foundSomething = NO;
+  int replaceCount = 0;
   if (doAll)
   {
     while (true)
@@ -1638,15 +1692,17 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
       if (result < 0)
         break;
 
-      foundSomething = YES;
+      replaceCount++;
       result = [ScintillaView directCall: self
                                  message: SCI_REPLACETARGET
                                   wParam: targetLength
                                   lParam: (sptr_t) replacement];
 
-      // The search in target call changes the target to the found text so we have to
-      // adjust it for the next search.
-      [self setGeneralProperty: SCI_SETTARGETEND value: endPosition];
+      // The replacement changes the target range to the replaced text. Continue after that til the end.
+      // The text length might be changed by the replacement so make sure the target end is the actual
+      // text end.
+      [self setGeneralProperty: SCI_SETTARGETSTART value: [self getGeneralProperty: SCI_GETTARGETEND]];
+      [self setGeneralProperty: SCI_SETTARGETEND value: [self getGeneralProperty: SCI_GETTEXTLENGTH]];
     }
   }
   else
@@ -1655,22 +1711,22 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
                                message: SCI_SEARCHINTARGET
                                 wParam: sourceLength
                                 lParam: (sptr_t) textToSearch];
-    foundSomething = (result < 0) ? NO : YES;
+    replaceCount = (result < 0) ? 0 : 1;
 
-    if (foundSomething)
+    if (replaceCount > 0)
     {
       result = [ScintillaView directCall: self
                                  message: SCI_REPLACETARGET
                                   wParam: targetLength
                                   lParam: (sptr_t) replacement];
 
-      // For a single replace we set the new selection to the replaced text.
+    // For a single replace we set the new selection to the replaced text.
     [self setGeneralProperty: SCI_SETSELECTIONSTART value: [self getGeneralProperty: SCI_GETTARGETSTART]];
     [self setGeneralProperty: SCI_SETSELECTIONEND value: [self getGeneralProperty: SCI_GETTARGETEND]];
     }
   }
   
-  return foundSomething;
+  return replaceCount;
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -1680,7 +1736,7 @@ static void notification(intptr_t windowid, unsigned int iMessage, uintptr_t wPa
                 bold: (BOOL) bold
                 italic: (BOOL) italic
 {
-  for (int i = 0; i < 32; i++)
+  for (int i = 0; i < 128; i++)
   {
     [self setGeneralProperty: SCI_STYLESETFONT
                    parameter: i
