@@ -23,11 +23,39 @@
 #import "Scintilla.h"
 #include "sqlide/sql_editor_be.h"
 
+@class MScintillaView;
+
+@interface NSObject(MScintillaViewDelegate)
+- (void)scintillaViewFindPanelClose:(MScintillaView*)editor;
+@end
+
+
 @interface MScintillaView : ScintillaView 
 {
+  IBOutlet NSView *mFindPanel;
+  IBOutlet NSSegmentedControl *mFindTypeSegmented;
+  IBOutlet NSTextField *mFindText;
+  IBOutlet NSTextField *mReplaceText;
+  IBOutlet NSTextField *mFindLabel;
+  IBOutlet NSSegmentedControl *mFindSegmented;
   Sql_editor::Ref mBackEnd;
+  BOOL mIgnoreCase;
+  BOOL mMatchWhole;
+  BOOL mWrapAround;
+  
+  id delegate;
 }
 
 - (void)setEditorBackEnd:(const Sql_editor::Ref&)be;
 
+- (NSView*)embedableFindPanel;
+- (void)focusFindPanel;
+- (void)enableReplaceInFindPanel: (BOOL)flag;
+
+- (IBAction)findActionClicked:(id)sender;
+
+- (void)setShowsSpecialCharacters:(BOOL)flag;
+
+- (id)delegate;
+
 @end
