@@ -32,13 +32,20 @@
 #import "WBPluginBase.h"
 #import "WBPluginEditorBase.h"
 #import "MCPPUtilities.h"
+#import "MTabSwitcher.h"
 #include "workbench/wb_context_ui.h"
 #include "model/wb_model_diagram_form.h"
 #include "model/wb_overview_physical.h"
 
 #import "WBMainWindow_Model.h"
 #include "base/string_utilities.h"
+#include "base/log.h"
+#include "base/notifications.h"
 
+#include "mforms/toolbar.h"
+#include "mforms/form.h"
+
+DEFAULT_LOG_DOMAIN("Workbench")
 
 @interface MyAutoreleasePool : NSAutoreleasePool
 {
@@ -53,6 +60,32 @@
 }
 @end
 
+class MacNotificationObserver : base::Observer
+{
+  WBMainWindow *mainWindow;
+public:
+  MacNotificationObserver(WBMainWindow *window)
+  : mainWindow(window)
+  {
+    base::NotificationCenter::get()->add_observer(this, "GNFormTitleDidChange");
+  }
+  
+  virtual ~MacNotificationObserver()
+  {
+    base::NotificationCenter::get()->remove_observer(this);
+  }
+  
+  virtual void handle_notification(const std::string &name, void *sender, base::NotificationInfo &info) 
+  {
+    if (name == "GNFormTitleDidChange")
+    {
+      WBBasePanel *panel = [mainWindow findMainPanelForUIForm: bec::UIForm::form_with_id(info["form"])];
+      if (panel)
+        [mainWindow setTitle: [NSString stringWithCPPString: info["title"]]
+                    forPanel: panel];
+    }
+  }
+};
 
 @implementation WBWindow
 
@@ -168,10 +201,8 @@ void setup_mforms_app(WBMainWindow *mwin);
 - (void)addBottomPanel:(WBBasePanel*)panel
 {
   WBBasePanel *mainPanel = [self selectedTopPanel];
-  if (mainPanel && [mainPanel isKindOfClass: [WBSplitPanel class]])
-  {
-    [(WBSplitPanel*)mainPanel addEditor: panel];
-  }
+  if ([mainPanel respondsToSelector: @selector(addEditor:)])
+    [(id)mainPanel addEditor: panel];
   else
     NSLog(@"No valid main-tab is selected to receive the bottom panel");
 }
@@ -236,6 +267,7 @@ void setup_mforms_app(WBMainWindow *mwin);
     "RefreshNothing",
     "RefreshSchemaList",
     "RefreshSchema",
+    "RefreshSchemaNoReload",
     "RefreshNewView",
     "RefreshSelection",
     "RefreshMessages",
@@ -325,6 +357,7 @@ void setup_mforms_app(WBMainWindow *mwin);
         [self switchToDiagramWithIdentifier: arg1.c_str()];
       break;
     }
+      
     case wb::RefreshViewName:
     {
       id diagramPanel= (id)arg2;
@@ -338,7 +371,7 @@ void setup_mforms_app(WBMainWindow *mwin);
         }
       }
       break;
-    }
+    } 
       
     case wb::RefreshSchemaList:
     case wb::RefreshSchema:
@@ -347,7 +380,10 @@ void setup_mforms_app(WBMainWindow *mwin);
       if ([panel respondsToSelector:@selector(refreshCatalogTree)])
         [panel refreshCatalogTree];
       break;
-    }      
+    }
+
+    case wb::RefreshSchemaNoReload:
+      break;
 
     case wb::RefreshZoom:
     {
@@ -548,15 +584,65 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
-- (BOOL)tabView:(NSTabView*)tabView itemIsClosable:(NSTabViewItem*)tabViewItem
+- (BOOL)tabView:(NSTabView*)tabView itemHasCloseButton:(NSTabViewItem*)tabViewItem
 {
-  if ([tabView indexOfTabViewItem: tabViewItem] == 0) // Home is not closable
+  id form= [_panels objectForKey:[tabViewItem identifier]]; 
+  bec::UIForm *uif = [form formBE];
+  if (uif && uif->get_form_context_name() == WB_CONTEXT_HOME_GLOBAL)
     return NO;
   return YES;
 }
 
 //--------------------------------------------------------------------------------------------------
 
+- (NSImage*)tabView:(NSTabView *)tabView iconForItem:(NSTabViewItem *)tabViewItem
+{
+  id form= [_panels objectForKey:[tabViewItem identifier]]; 
+  bec::UIForm *uif = [form formBE];
+  if (uif && uif->get_form_context_name() == WB_CONTEXT_HOME_GLOBAL)
+    return _homeIcon;
+  return nil;
+}
+
+//--------------------------------------------------------------------------------------------------
+
+- (IBAction)handleMenuAction:(id)sender
+{
+  // Close Other Tabs Like This from the main tab
+  if ([sender tag] == 1002)
+  {
+    id form= [_panels objectForKey:[[tabSwitcher clickedItem] identifier]];
+    if (form && [form formBE])
+    {
+      std::string context_name;
+      if ([form isKindOfClass: [WBMFormsPluginPanel class]])
+        context_name = [form formBE]->get_form_context_name();
+      for (NSTabViewItem *item in [[topTabView tabViewItems] reverseObjectEnumerator])
+      {
+        if (item != [tabSwitcher clickedItem] && [self tabView: topTabView itemHasCloseButton: item])
+        {
+          id itemForm = [_panels objectForKey: [item identifier]];
+          if (context_name.empty())
+          {
+            if ([itemForm isKindOfClass: [form class]])
+              [tabSwitcher closeTabViewItem: item];
+          }
+          else
+          {
+            bec::UIForm *itemFormBE = [itemForm formBE];
+            if (itemFormBE && [itemForm isKindOfClass: [WBMFormsPluginPanel class]]
+                && itemFormBE->get_form_context_name() == context_name)
+              [tabSwitcher closeTabViewItem: item];
+          }
+        }
+      }
+    }
+    [tabSwitcher setNeedsDisplay: YES];
+  }
+}
+
+//--------------------------------------------------------------------------------------------------
+
 - (BOOL)closePanel:(WBBasePanel*)panel
 {
   id identifier = [[[panel identifier] retain] autorelease];
@@ -838,6 +924,19 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 //--------------------------------------------------------------------------------------------------
 
+- (WBBasePanel*)findMainPanelForUIForm: (bec::UIForm*)form
+{
+  for (NSTabViewItem *item in [topTabView tabViewItems])
+  {
+    id panel = [_panels objectForKey: [item identifier]];
+    if ([panel formBE] == form)
+      return panel;
+  }
+  return nil;  
+}
+                        
+//--------------------------------------------------------------------------------------------------
+
 - (void)closeEditorsMatching: (NSString*)identifier
 {  
   for (NSString *key in [_panels keyEnumerator])
@@ -951,8 +1050,10 @@ void setup_mforms_app(WBMainWindow *mwin);
   
   i = [topTabView indexOfTabViewItemWithIdentifier: [panel identifier]];
   if (i >= 0 && i != NSNotFound)
+  {
     [[topTabView tabViewItemAtIndex: i] setLabel: title];
-  //XXX
+    [[self window] display];
+  }
 }
 
 //--------------------------------------------------------------------------------------------------
@@ -970,10 +1071,14 @@ void setup_mforms_app(WBMainWindow *mwin);
     
     _defaultMainMenu = [[NSApp mainMenu] retain];
     
+    _homeIcon = [[NSImage imageNamed: @"WB_Home_mac.png"] retain];
+    
     [[NSNotificationCenter defaultCenter] addObserver: self
                                              selector: @selector(applicationWillTerminate:)
                                                  name: NSApplicationWillTerminateNotification
-                                               object: nil];    
+                                               object: nil];
+	
+	_backendObserver = new MacNotificationObserver(self);
   }
   return self;
 }
@@ -982,6 +1087,7 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)load
 {
+  log_info("Loading main window\n");
   if (![NSBundle loadNibNamed:@"MainWindow" owner:self])
     NSLog(@"Failed loading nib");
 }
@@ -990,6 +1096,8 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)awakeFromNib
 {
+  [tabSwitcher setTabStyle: MMainTabSwitcher];
+  
   [statusBarText setStringValue:@""];
 }
 
@@ -1007,12 +1115,14 @@ void setup_mforms_app(WBMainWindow *mwin);
 
 - (void)dealloc
 {
+  delete _backendObserver;
+	
   [_closedTopTabs release];
   [_closedBottomTabs release];
   
   [_panels release];
   [_physicalOverview release];
-	
+  
   [super dealloc];
 }
 
@@ -1038,7 +1148,7 @@ void setup_mforms_app(WBMainWindow *mwin);
 //--------------------------------------------------------------------------------------------------
 
 // mforms integration
-#include <mforms/mforms.h>
+#import <mforms/../cocoa/MFMForms.h>
 
 static void dock_view(mforms::App *app, mforms::AppView *view, const std::string &position)
 {
