@@ -388,6 +388,9 @@ namespace Aga.Controls.Tree
 
 		private void SafeUpdateScrollBars()
 		{
+      if (!IsHandleCreated)
+        return;
+
 			if (InvokeRequired)
 				Invoke(new MethodInvoker(UpdateScrollBars));
 			else
@@ -555,7 +558,11 @@ namespace Aga.Controls.Tree
 
 		public void FullUpdate()
 		{
-      if (InvokeRequired)
+      // ml: gets called from the background thread at times when the tree is already gone mostly.
+      if (IsDisposed || Disposing)
+        return;
+
+      if (IsHandleCreated && InvokeRequired)
         Invoke(new MethodInvoker(UnsafeFullUpdate));
       else
         UnsafeFullUpdate();
