@@ -1,4 +1,4 @@
-/**
+/*
  * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
@@ -22,7 +22,6 @@ using System.Collections.Generic;
 using System.Drawing;
 using System.IO;
 using System.Reflection;
-using System.Threading;
 using System.Windows.Forms;
 
 using Aga.Controls.Tree;
@@ -35,8 +34,8 @@ using MySQL.GUI.Mdc;
 using MySQL.GUI.Workbench.Plugins;
 using MySQL.GUI.Workbench.Properties;
 using MySQL.Utilities;
-using MySQL.Workbench;
 using MySQL.Utilities.SysUtils;
+using MySQL.Workbench;
 
 namespace MySQL.GUI.Workbench
 {
@@ -44,7 +43,7 @@ namespace MySQL.GUI.Workbench
   {
     #region Member Variables
 
-    private String[] resourcePaths = {
+    static private String[] resourcePaths = {
       "",
       "images/ui",
       "images/icons",
@@ -66,9 +65,6 @@ namespace MySQL.GUI.Workbench
     // Special Forms
     private ModelOverviewForm workbenchPhysicalOverviewForm = null;
 
-    // TODO: this should go once the entire menu/toolbar handling is in the backend.
-    private WorkbenchToolbarManager workbenchToolbarManager = null;
-
     private Form findForm = null;
 
     private bool shuttingDown = false;
@@ -96,8 +92,6 @@ namespace MySQL.GUI.Workbench
       wbContext = WbContext;
       grtManager = wbContext.get_grt_manager();
 
-      workbenchToolbarManager = new WorkbenchToolbarManager(wbContext);
-
       tabImageList = new ImageList();
       tabImageList.ColorDepth = ColorDepth.Depth32Bit;
       tabImageList.ImageSize = new Size(16, 16);
@@ -139,6 +133,11 @@ namespace MySQL.GUI.Workbench
       mainStatusStrip.Items.Add(host);
     }
 
+    void Destroy()
+    {
+      // All documents are closed in Form_Closing.
+    }
+
     #endregion
 
     #region Implement Interfaces
@@ -320,17 +319,7 @@ namespace MySQL.GUI.Workbench
           Close();
           break;
         case AppCommand.AppGetResourcePath:
-          // Try to find the given file in known locations. For now we only search the image paths.
-          // That will get improved later to find any resource in any location (including resource DLLs etc.).
-          result = "";
-          if (File.Exists(str))
-            return str;
-          foreach (String path in resourcePaths)
-            if (File.Exists(path + "/" + str))
-            {
-              result = path + "/" + str;
-              break;
-            }
+          result = GetIconPath(str);
           break;
         case AppCommand.AppSetStatusText:
           StatusBarText = str;
@@ -376,40 +365,11 @@ namespace MySQL.GUI.Workbench
           {
             // Open only diagrams which were open when this model was closed last time.
             ModelDiagramForm modelDiagramForm = canvas.GetOwnerForm() as ModelDiagramForm;
-            if (modelDiagramForm != null && !((ModelViewForm)modelDiagramForm.BackendClass).is_closed())
+            if (modelDiagramForm != null && !modelDiagramForm.DiagramWrapper.is_closed())
               DockDocument(modelDiagramForm, true, true);
           }
           break;
         }
-        case RefreshType.RefreshViewName:
-        {
-          BaseWindowsCanvasView canvas = BaseWindowsCanvasView.GetFromFixedId(ptr);
-          if (canvas != null)
-          {
-            ModelDiagramForm modelDiagramForm = canvas.GetOwnerForm() as ModelDiagramForm;
-
-            if (modelDiagramForm != null)
-              modelDiagramForm.TabText = str;
-          }
-          break;
-        }
-
-        case RefreshType.RefreshOverviewNodeChildren:
-          if (workbenchPhysicalOverviewForm != null)
-            workbenchPhysicalOverviewForm.RefreshNodeChildren(new NodeId(str));
-          foreach (ITabDocument content in contentTabControl.Documents)
-          {
-            if (content is DbSqlEditor)
-            {
-              DbSqlEditor dbSqlEditor = content as DbSqlEditor;
-              if (dbSqlEditor.LivePhysicalOverviewBE.matches_handle(ptr))
-              {
-                dbSqlEditor.LivePhysicalOverview.RefreshNodeChildren(new NodeId(str));
-                break;
-              }
-            }
-          }
-          break;
 
         case RefreshType.RefreshNewModel:
           ShowPhysicalOverviewForm(new NodeId(str)); // Make sure the physical overview exists.
@@ -426,8 +386,10 @@ namespace MySQL.GUI.Workbench
 
         case RefreshType.RefreshSchema:
         case RefreshType.RefreshSchemaList:
+        case RefreshType.RefreshSchemaNoReload:
         case RefreshType.RefreshSelection:
         case RefreshType.RefreshZoom:
+        case RefreshType.RefreshOverviewNodeChildren:
           ForwardRefreshToActivDocument(refresh, str, ptr);
           break;
 
@@ -569,8 +531,6 @@ namespace MySQL.GUI.Workbench
                 // Make sure form is removed from GRT editor list when closed
                 pluginForm.EditorPlugin.FormClosed += new FormClosedEventHandler(PluginFormClosed);
 
-                InstallFocusChangeHandlers(pluginForm);
-
                 // Get fixed ptr
                 ptr = pluginForm.GetFixedPtr();
 
@@ -757,7 +717,7 @@ namespace MySQL.GUI.Workbench
           return (activeControl as Scintilla).Selection.Length > 0;
         else
           if (activeControl is GridView)
-            return (activeControl as GridView).CanCopy();
+            return (activeControl as GridView).CanCopy;
           else
             return wbContext.edit_can_copy();
     }
@@ -873,54 +833,71 @@ namespace MySQL.GUI.Workbench
 
     private void EditFind()
     {
-      // TODO: Remove once the simple search code moved to the individual pages.
+      // TODO: unify search/replace calls, currently they take two paths (via main menu and
+      // via SQL editor partial ui refresh (if an editor is active).
+      Control activeControl = ControlUtilities.GetLeafActiveControl(this);
+      if (activeControl is Scintilla)
+        (activeControl as Scintilla).Commands.Execute(BindableCommand.ShowFind);
+      else
+      {
+        // Not an editor control. So try focusing the search box in the toolbar (if there's any).
+        IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+        if (wbDocument != null)
+          wbContext.focus_search_box(wbDocument.BackendForm);
+      }
     }
 
     private bool EditCanFind()
     {
-      //return (workbenchToolbarManager.searchBox != null);
-      return false; // TODO: The simple search code should now go to the individual pages.
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == null)
+        return false;
+
+      if (wbDocument is DbSqlEditor)
+      {
+        Control activeControl = ControlUtilities.GetLeafActiveControl(this);
+        if (activeControl is Scintilla)
+          return true;
+        return false;
+      }
+
+      return true;
     }
 
     private void EditFindReplace()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is MySQL.Grt.Db.Sql.SqlEditor)
-      {
-        MySQL.Grt.Db.Sql.SqlEditor editor = activeControl as MySQL.Grt.Db.Sql.SqlEditor;
-
-        editor.ShowFindReplaceDialog();
-      }
+      if (activeControl is Scintilla)
+        (activeControl as Scintilla).Commands.Execute(BindableCommand.ShowReplace);
     }
 
     private bool EditCanFindReplace()
     {
       Control activeControl = ControlUtilities.GetLeafActiveControl(this);
-      if (activeControl is MySQL.Grt.Db.Sql.SqlEditor)
+      if (activeControl is Scintilla)
         return true;
       return false;
     }
 
     private void SearchActiveTab()
     {
-      // TODO: remove this once the code moved to the individual pages.
-      /*
-      String text= workbenchToolbarManager.lastSearchString;
-      if (text != "")
+      IWorkbenchDocument wbDocument = contentTabControl.ActiveDocument as IWorkbenchDocument;
+      if (wbDocument == null)
+        return;
+
+      string searchString = wbContext.get_search_string(wbDocument.BackendForm);
+      if (searchString != "")
       {
-        if (!wbContext.try_searching_diagram(text))
+        if (!wbContext.try_searching_diagram(searchString))
         {
           // if active main tab is Overview, search it
           if (contentTabControl.ActiveDocument == workbenchPhysicalOverviewForm)
-            workbenchPhysicalOverviewForm.SearchAndFocusNode(text);
-          else
-            if (contentTabControl.ActiveDocument is DbSqlEditor)
-              (contentTabControl.ActiveDocument as DbSqlEditor).SearchText(text);
+            workbenchPhysicalOverviewForm.SearchAndFocusNode(searchString);
         }
       }
-      */
     }
-#endregion
+    
+    #endregion
 
     private void RegisterCommands()
     {
@@ -941,6 +918,7 @@ namespace MySQL.GUI.Workbench
       commands.Add("closetab");
       commands.Add("help_version_check");
       commands.Add("wb.sidebarHide");
+      commands.Add("close_tab");
 
       wbContext.add_frontend_commands(commands);
 
@@ -979,6 +957,10 @@ namespace MySQL.GUI.Workbench
       {
         CloseActiveTab();
       }
+      else if (command == "close_tab")
+      {
+        contentTabControl.CloseTabPage(contentTabControl.SelectedTab);
+      }
       else if (command.Equals("help_version_check"))
         Program.CheckForNewVersion();
       else if (command.Equals("wb.sidebarHide"))
@@ -1041,22 +1023,17 @@ namespace MySQL.GUI.Workbench
       PageSettingsForm form = new PageSettingsForm(wbContext);
 
       form.ShowDialog();
-
       form.Dispose();
     }
-
-    
+        
     private void ShowDiagramOptionsForm()
     {
       DiagramOptionsForm form = new DiagramOptionsForm(wbContext);
 
       form.ShowDialog();
-
       form.Close();
-      form.Dispose();
     }
 
-
     public String ShowFileDialog(String type, String title, String extensions)
     {
       FileDialog dialog;
@@ -1111,6 +1088,25 @@ namespace MySQL.GUI.Workbench
 
     #region Standard Forms Handling
 
+    /// <summary>
+    /// Try to find the given file in known locations. For now we only search the image paths.
+    /// That will get improved later to find any resource in any location (including resource DLLs etc.).
+    /// </summary>
+    /// <param name="name"></param>
+    static public string GetIconPath(string name)
+    {
+      string result = "";
+      if (File.Exists(name))
+        return name;
+      foreach (String path in resourcePaths)
+        if (File.Exists(path + "/" + name))
+        {
+          result = path + "/" + name;
+          break;
+        }
+      return result;
+    }
+
     private void grtOutputImg_Click(object sender, EventArgs e)
     {
       wbContext.show_output();
@@ -1177,7 +1173,7 @@ namespace MySQL.GUI.Workbench
         if (document is ModelDiagramForm)
         {
           ModelDiagramForm form = document as ModelDiagramForm;
-          ((ModelViewForm)form.BackendClass).set_closed(false);
+          ((ModelDiagramFormWrapper)form.BackendForm).set_closed(false);
         }
       }
     }
@@ -1190,16 +1186,57 @@ namespace MySQL.GUI.Workbench
         if (document is ModelDiagramForm)
         {
           ModelDiagramForm form = document as ModelDiagramForm;
-          ((ModelViewForm)form.BackendClass).set_closed(true);
+          ((ModelDiagramFormWrapper)form.BackendForm).set_closed(true);
         }
       }
     }
 
+    private void SetEnableForMenuItem(ToolStripItem item, bool enabled)
+    {
+      item.Enabled = enabled;
+      if (item is ToolStripMenuItem)
+      {
+        foreach (ToolStripItem subitem in (item as ToolStripMenuItem).DropDownItems)
+          SetEnableForMenuItem(subitem, enabled);
+      }
+    }
+
     private void tabControl_SelectedIndexChanged(object sender, EventArgs e)
     {
       // Reset status bar to clean up any previously done action message.
       StatusBarText = "Ready";
 
+      // Disable embedded menus for all inactive pages.
+      ITabDocument[] documents = contentTabControl.DocumentsToArray();
+      for (int i = 0; i < documents.Length; i++)
+      {
+        AppViewDockContent content = documents[i] as AppViewDockContent;
+        if (content != null)
+        {
+          MenuStrip menu = content.GetAppView().GetMenuBar();
+          if (menu != null)
+          {
+            menu.Enabled = (contentTabControl.SelectedIndex == i);
+            foreach (ToolStripItem item in menu.Items)
+              SetEnableForMenuItem(item, menu.Enabled);
+          }
+        }
+        else
+        {
+          IWorkbenchDocument wbDocument = documents[i] as IWorkbenchDocument;
+          if (wbDocument != null)
+          {
+            MenuStrip menu = wbContext.menu_for_form(wbDocument.BackendForm);
+            if (menu != null)
+            {
+              menu.Enabled = (contentTabControl.SelectedIndex == i);
+              foreach (ToolStripItem item in menu.Items)
+                SetEnableForMenuItem(item, menu.Enabled);
+            }
+          }
+        }
+      }
+      
       // For the glass effect behind the tabs (and the proper content drawing) we need the
       // height of the tabs as well as menu and toolbar of the active tab.
       int topAreaHeight = 0;
@@ -1224,19 +1261,19 @@ namespace MySQL.GUI.Workbench
             {
               ModelDiagramForm form = activeDocument as ModelDiagramForm;
 
-              wbContext.set_active_form(form.BackendClass);
+              wbContext.set_active_form(form.BackendForm);
 
               form.FocusCanvasControl();
             }
             else
             {
-              wbContext.set_active_form(wbDoc.BackendClass);  
+              wbContext.set_active_form(wbDoc.BackendForm);  
             }
           }
           else if (activeDocument is Plugins.DockablePlugin)
           {
             Plugins.DockablePlugin form = activeDocument as Plugins.DockablePlugin;
-            wbContext.set_active_form(form.BackendClass);
+            wbContext.set_active_form(form.BackendForm);
           }
           else if (activeDocument is UIForm)
           {
@@ -1277,7 +1314,6 @@ namespace MySQL.GUI.Workbench
       if (plugin != null)
       {
         wbContext.close_gui_plugin(plugin.GetFixedPtr());
-        plugin.ReleaseHandle();
       }
 
       Program.CollectGarbageOnIdle();
@@ -1324,13 +1360,13 @@ namespace MySQL.GUI.Workbench
         if (!contentTabControl.HasDocument(document))
         {
           IWorkbenchDocument wbDocument = document as IWorkbenchDocument;
-          SetupDockLayout(document, (wbDocument != null) ? wbDocument.BackendClass : null);
+          SetupDockLayout(document, (wbDocument != null) ? wbDocument.BackendForm : null);
           index = contentTabControl.AddDocument(document);
 
           InstallFocusChangeHandlers(document as Control);
         }
 
-        if (index == 0) // The main tab.
+        if (index == 0) // Replace the title by an icon.
         {
           contentTabControl.TabPages[index].ImageIndex = 0;
           contentTabControl.SetCloseButtonVisibility(0, FlatTabControl.CloseButtonVisiblity.HideButton);
@@ -1346,16 +1382,13 @@ namespace MySQL.GUI.Workbench
       {
         ITabDocument host = contentTabControl.ActiveDocument;
         if (host is ModelDiagramForm)
-        {
-          ModelDiagramForm form = host as ModelDiagramForm;
-          form.DockDocument(document, activate);
-        }
+          (host as ModelDiagramForm).DockDocument(document, activate);
         else
           if (host is ModelOverviewForm)
-          {
-            ModelOverviewForm form = host as ModelOverviewForm;
-            form.DockDocument(document, activate);
-          }
+            (host as ModelOverviewForm).DockDocument(document, activate);
+          else
+            if (host is DbSqlEditor)
+              (host as DbSqlEditor).DockDocument(document, activate);
       }
     }
 
@@ -1376,6 +1409,9 @@ namespace MySQL.GUI.Workbench
         else
           if (host is ModelOverviewForm)
             (host as ModelOverviewForm).UndockDocument(document);
+          else
+            if (host is DbSqlEditor)
+              (host as DbSqlEditor).UndockDocument(document);
       }
     }
 
@@ -1398,40 +1434,8 @@ namespace MySQL.GUI.Workbench
     private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
     {
       shuttingDown = true;
-      Hide();
 
       e.Cancel = !wbContext.request_quit();
-      ITabDocument[] documents = contentTabControl.DocumentsToArray();
-
-      /* This code is doing work which is already done in request_quit(), so it can be 
-       * removed once we can be sure that works in any circumstance.
-       *
-      if (!e.Cancel)
-      {
-        foreach (ITabDocument document in documents)
-        {
-          if (document is DbSqlEditor)
-          {
-            DbSqlEditor dbSqlEditor = document as DbSqlEditor;
-            e.Cancel = !dbSqlEditor.Backend.can_close();
-            if (e.Cancel)
-              break;
-          }
-        }
-      }
-
-      if (!e.Cancel && wbContext.has_unsaved_changes())
-      {
-        DialogResult result = MessageBox.Show("Do you want to save changes to the document before closing?\n\n" +
-                        "If you don't save your changes, they will be lost.",
-                        "Close Workbench", MessageBoxButtons.YesNoCancel, MessageBoxIcon.Exclamation);
-        if (result == DialogResult.Cancel)
-          e.Cancel = true;
-        else 
-          if (result != DialogResult.No && !wbContext.save_changes())
-            e.Cancel = true;
-      }
-      */
 
       // Reset termination flag, in case it was set.
       if (e.Cancel)
@@ -1444,10 +1448,13 @@ namespace MySQL.GUI.Workbench
       RemoveFocusChangeHandlers(this);
       SaveFormState();
 
+      Hide();
       // Go through each workbench document and close it unconditionally.
       contentTabControl.SuspendLayout();
       try
       {
+        ITabDocument[] documents = contentTabControl.DocumentsToArray();
+
         // Close all documents (editors implement the document interface too, so we get them
         // with that loop as well).
         for (int i = documents.Length - 1; i >= 0; i--)
@@ -1461,6 +1468,7 @@ namespace MySQL.GUI.Workbench
           else
             documents[i].Close();
         }
+        workbenchPhysicalOverviewForm = null;
       }
       finally
       {
@@ -1598,7 +1606,7 @@ namespace MySQL.GUI.Workbench
       if (FocusChangeHandler == null)
         FocusChangeHandler= new EventHandler(FocusChanged);
 
-      // Recursively install event handler for GotFocus and LostFocus.
+      // Recursively install event handler for GotFocus.
       control.GotFocus -= FocusChangeHandler;
       control.GotFocus += FocusChangeHandler;
 
@@ -1613,6 +1621,9 @@ namespace MySQL.GUI.Workbench
       e.canClose = true;
 
       ITabDocument document = (sender as FlatTabControl).DocumentFromPage(e.page);
+      if (document == null)
+        return;
+
       if (document is DbSqlEditor)
         e.canClose = (document as DbSqlEditor).Backend.can_close();
       if (document is Plugins.DockablePlugin)
@@ -1622,7 +1633,7 @@ namespace MySQL.GUI.Workbench
       }
       else
         if (document is IWorkbenchDocument)
-          e.canClose = (document as IWorkbenchDocument).BackendClass.can_close();
+          e.canClose = (document as IWorkbenchDocument).BackendForm.can_close();
         else
           if (document is MySQL.Forms.AppViewDockContent)
           {
@@ -1649,7 +1660,7 @@ namespace MySQL.GUI.Workbench
         PluginFormClosed(sender, null);
       else
         if (document is IWorkbenchDocument)
-          (document as IWorkbenchDocument).BackendClass.close();
+          (document as IWorkbenchDocument).BackendForm.close();
         else
           if (document is MySQL.Forms.AppViewDockContent)
           {
@@ -1767,10 +1778,85 @@ namespace MySQL.GUI.Workbench
         }
       }
     }
-    
+
+    private Bitmap logo = null;
+
     void menuPanel_PaintBackground(object sender, PaintEventArgs e)
     {
       ToolStripHelper.MenuPanelPaint(e.Graphics, (sender as Control).ClientRectangle);
+
+      if (logo == null && File.Exists("images/ui/oracle_logo_white.png"))
+        logo = new Bitmap("images/ui/oracle_logo_white.png");
+
+      if (logo != null)
+      {
+        int x = (sender as Control).Width - logo.Width - 8;
+        int y = 6;
+        e.Graphics.DrawImageUnscaled(logo, new Point(x, y));
+      }
+    }
+
+    private void tabsContextMenuItemClick(object sender, EventArgs e)
+    {
+      TabPage page = contentTabControl.SelectedTab;
+      if (page == null)
+        return;
+
+      ToolStripMenuItem item = sender as ToolStripMenuItem;
+      int clickedTab = (int)item.Owner.Tag;
+      switch (item.Tag as string)
+      {
+        case "0": // Close page.
+          contentTabControl.CloseTabPage(page);
+          break;
+        case "1": // Close all editors but the clicked one.
+          TabControl.TabPageCollection pages = contentTabControl.TabPages;
+          for (int i = 0; i < pages.Count; i++)
+          {
+            // Do not close the home screen.
+            if (i == 0 || pages[i] == page)
+              continue;
+
+            contentTabControl.CloseTabPage(pages[i]);
+          }
+          break;
+        case "2": // Close tabs of the same type.
+          ITabDocument document = contentTabControl.DocumentFromIndex(clickedTab);
+          Type classToClose = document.GetType();
+          for (int i = contentTabControl.TabCount - 1; i >= 0; i--)
+          {
+            if (contentTabControl.DocumentFromIndex(i).GetType() == classToClose)
+              contentTabControl.CloseTabPage(contentTabControl.TabPages[i]);
+          }
+          break;
+      }
+    }
+
+    private void contentTabControl_MouseClick(object sender, MouseEventArgs e)
+    {
+      switch (e.Button)
+      {
+        case MouseButtons.Right:
+          {
+            int clickedIndex = contentTabControl.TabIndexFromPosition(e.Location);
+            if (clickedIndex < 0)
+              return;
+
+            closeTabMenuItem.Enabled = clickedIndex > 0;
+            closeOtherTabsMenuItem.Enabled = contentTabControl.TabCount > 1;
+
+            ITabDocument document = contentTabControl.DocumentFromIndex(clickedIndex);
+            closeTabsOfSameTypeItem.Enabled = document is DbSqlEditor || document is ModelDiagramForm
+              || document is ModelOverviewForm;
+
+            // Keep the found index for later handling in the item click handler.
+            tabsContextMenuStrip.Tag = clickedIndex;
+
+            Point p = contentTabControl.PointToScreen(e.Location);
+            tabsContextMenuStrip.Show(p);
+          }
+          break;
+      }
     }
 
   }
