@@ -33,7 +33,7 @@ namespace MySQL.Grt.Db.Sql
 {
   public class SqlEditor : Scintilla
   {
-    private Sql_editor sqlEditorBE;
+    private SqlEditorWrapper sqlEditorBE;
     private GrtManager grtManager;
 
     private Timer timer;
@@ -68,7 +68,7 @@ namespace MySQL.Grt.Db.Sql
       set { grtManager = value; }
     }
 
-    public Sql_editor BE
+    public SqlEditorWrapper BE
     {
       get { return sqlEditorBE; }
       set
@@ -83,7 +83,6 @@ namespace MySQL.Grt.Db.Sql
         sqlEditorBE.sql_parser_err_cb(ProcessSqlError);
         sqlEditorBE.set_insert_text_delegate(DoInsertText);
         sqlEditorBE.set_replace_selected_text_delegate(DoReplaceSelectedText);
-        sqlEditorBE.set_do_search_delegate(DoTextSeach);
         sqlEditorBE.set_current_statement_delegate(GetCurrentSqlStatement);
         sqlEditorBE.set_change_selected_range_delegate(ChangeSelectedRange);
         sqlEditorBE.set_change_cursor_pos_delegate(ChangeCursorPos);
@@ -92,14 +91,17 @@ namespace MySQL.Grt.Db.Sql
 
         SuspendLayout();
 
-        // customized parameters
+        // Customized parameters.
         ConfigurationManager.Language = sqlEditorBE.string_option("ConfigurationManager.Language");
+        Lexing.SetKeywords(0, sqlEditorBE.get_keywords(KeywordList.GeneralKeywords));
+        Lexing.SetKeywords(3, sqlEditorBE.get_function_names());
+        Lexing.SetKeywords(5, sqlEditorBE.get_keywords(KeywordList.ProcedureKeywords));
+        Lexing.SetKeywords(6, sqlEditorBE.get_keywords(KeywordList.UserKeywords));
+        Lexing.SetKeywords(7, sqlEditorBE.get_keywords(KeywordList.ClientKeywords));
+
         Indentation.IndentWidth = sqlEditorBE.int_option("Indentation.IndentWidth");
         Indentation.TabWidth = sqlEditorBE.int_option("Indentation.TabWidth");
         Indentation.UseTabs = sqlEditorBE.int_option("Indentation.UseTabs") != 0;
-        //! todo: virtualize the rest params
-
-        //EndOfLine.IsVisible = true;
 
         ResumeLayout();
       }
@@ -378,22 +380,16 @@ namespace MySQL.Grt.Db.Sql
     public int ProcessSqlError(int errTokLine, int errTokLinePos, int errTokLen, String msg)
     {
       errTokLine -= 1;
-      try
-      {
-        errTokLinePos += Lines[errTokLine].Range.Start;
-        ManagedRange range = new ManagedRange(errTokLinePos, errTokLinePos + errTokLen, this);
+      errTokLinePos += Lines[errTokLine].Range.Start;
+      ManagedRange range = new ManagedRange(errTokLinePos, errTokLinePos + errTokLen, this);
         
-        SuspendLayout();
-        range.SetIndicator(0);
-        range.SetIndicator(1);
-        Lines[errTokLine].AddMarker(1);
-        ResumeLayout();
+      range.SetIndicator(0);
+      Lines[errTokLine].AddMarker(1);
+
+      messages.Add(msg);
+      messagesRanges.Add(range);
+      ManagedRanges.Add(range);
 
-        messages.Add(msg);
-        messagesRanges.Add(range);
-        ManagedRanges.Add(range);
-      }
-      catch (Exception) { }
       return 0;
     }
 
@@ -522,99 +518,5 @@ namespace MySQL.Grt.Db.Sql
       return 0;
     }
 
-    public void ShowFindReplaceDialog()
-    {
-      // Scintilla.NET already has an implementation of the search/replace dialog and all the
-      // usual features (including search/replace in the current selection etc.). So we use that
-      // instead of our own dialog.
-      //sqlEditorBE.show_find_replace_dialog();
-      Commands.Execute(BindableCommand.ShowReplace);
-    }
-
-    bool DoTextSeach(String searchText, String replaceText, int flags)
-    {
-      if (searchText.Length == 0)
-        return false;
-
-      Sql_editor.SearchFlags searchFlags = (Sql_editor.SearchFlags) flags;
-      SearchFlags scintillaFlags = SearchFlags.Empty;
-
-      bool previous = (searchFlags & Sql_editor.SearchFlags.SearchPrevious) != 0;
-      bool useRegEx = (searchFlags & Sql_editor.SearchFlags.SearchUseRegularExpression) != 0;
-      bool replace = (searchFlags & Sql_editor.SearchFlags.SearchDoReplace) != 0;
-      bool all = (searchFlags & Sql_editor.SearchFlags.SearchAll) != 0;
-
-      Range searchRange = new Range(0, RawText.Length, this);
-      if ((searchFlags & Sql_editor.SearchFlags.SearchMatchCase) != 0)
-        scintillaFlags |= SearchFlags.MatchCase;
-      if ((searchFlags & Sql_editor.SearchFlags.SearchMatchWholeWord) != 0)
-        scintillaFlags |= SearchFlags.WholeWord;
-
-      Range result = null;
-      bool canClose = false;
-      if (useRegEx)
-      {
-        Regex expression = new Regex(searchText);
-        if (replace)
-        {
-          if (all)
-          {
-            FindReplace.ReplaceAll(expression, replaceText);
-            canClose = true;
-          }
-          else
-          {
-            // There is no overload to replace with RE search, so we have to handle that ourselves.
-            if (previous)
-              result = FindReplace.FindPrevious(expression, true);
-            else
-              result = FindReplace.FindNext(expression, true);
-
-            if (result != null)
-            {
-              result.Text = replaceText;
-              result.End = result.Start + replaceText.Length;
-            }
-          }
-        }
-        else
-          if (previous)
-            result = FindReplace.FindPrevious(expression, true);
-          else
-            result = FindReplace.FindNext(expression, true);
-      }
-      else
-      {
-        if (replace)
-        {
-          if (all)
-          {
-            FindReplace.ReplaceAll(searchText, replaceText, scintillaFlags);
-            canClose = true;
-          }
-          else
-          {
-            if (previous)
-              result = FindReplace.ReplacePrevious(searchText, replaceText, true, scintillaFlags);
-            else
-              result = FindReplace.ReplaceNext(searchText, replaceText, true, scintillaFlags);
-          }
-        }
-        else
-        {
-          if (previous)
-            result = FindReplace.FindPrevious(searchText, true, scintillaFlags);
-          else
-            result = FindReplace.FindNext(searchText, true, scintillaFlags);
-        }
-      }
-      if (result != null)
-      {
-        canClose = true;
-        result.Select();
-      }
-
-      return canClose;
-    }
   }
 }
