@@ -102,8 +102,8 @@ void GtkCanvas::on_map()
     return;
   }
 
-  _canvas->signal_viewport_changed().connect(sigc::mem_fun(this, &GtkCanvas::canvas_view_viewport_changed));
-  _canvas->signal_repaint().connect(sigc::mem_fun(this, &GtkCanvas::canvas_view_needs_repaint));
+  scoped_connect(_canvas->signal_viewport_changed(),sigc::mem_fun(this, &GtkCanvas::canvas_view_viewport_changed));
+  scoped_connect(_canvas->signal_repaint(),sigc::mem_fun(this, &GtkCanvas::canvas_view_needs_repaint));
 
   _canvas->repaint();
 }
