@@ -29,8 +29,8 @@ using namespace MySQL::Geometry;
 StraightLineLayouter::StraightLineLayouter(Connector *sconn, Connector *econn)
    : _start_conn(sconn), _end_conn(econn)
 {
-  sconn->set_update_handler(sigc::mem_fun(this, &StraightLineLayouter::connector_changed));
-  econn->set_update_handler(sigc::mem_fun(this, &StraightLineLayouter::connector_changed));
+  sconn->set_update_handler(boost::bind(&StraightLineLayouter::connector_changed, this, _1));
+  econn->set_update_handler(boost::bind(&StraightLineLayouter::connector_changed, this, _1));
 
   _start= sconn->get_position();
   _end= econn->get_position();
@@ -131,6 +131,6 @@ void StraightLineLayouter::connector_changed(Connector *conn)
     }
   }
   if (changed)
-    _changed.emit();
+    _changed();
 }
 
