@@ -1,15 +1,26 @@
-//
-//  MFUtilities.mm
-//  MySQLWorkbench
-//
-//  Created by Alfredo Kojima on 5/Mar/09.
-//  Copyright 2009 Sun Microsystems Inc. All rights reserved.
-//
+/* 
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
 
 #import "MFUtilities.h"
 #import "MHudController.h"
 
-#include "mforms/mforms.h"
+#import "MFMForms.h"
 #import "MFBase.h"
 #include <stdexcept>
 
@@ -45,7 +56,7 @@ static int util_show_message_with_checkbox(const std::string &title, const std::
     [[alert suppressionButton] setTitle: wrap_nsstring(cb_message)];
   
   if (!ok.empty())
-    [[[alert buttons] lastObject] setTitle: wrap_nsstring(ok)];
+    [[alert addButtonWithTitle: wrap_nsstring(ok)] setTag: NSAlertDefaultReturn];
   
   if (!cancel.empty())
     [[alert addButtonWithTitle: wrap_nsstring(cancel)] setTag: NSAlertOtherReturn];
@@ -82,7 +93,10 @@ static std::string util_get_clipboard_text()
 
 static void util_open_url(const std::string &url)
 {
-  [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString: [NSString stringWithUTF8String: url.c_str()]]];
+  if (g_file_test(url.c_str(), G_FILE_TEST_EXISTS) || (!url.empty() && url[0] == '/'))
+    [[NSWorkspace sharedWorkspace] openURL:[NSURL fileURLWithPath: [NSString stringWithUTF8String: url.c_str()]]];
+  else
+    [[NSWorkspace sharedWorkspace] openURL:[NSURL URLWithString: [NSString stringWithUTF8String: url.c_str()]]];
 }
 
 
@@ -106,19 +120,19 @@ static std::string get_special_folder(mforms::FolderType type)
   boost::function<bool ()> *callback;
 }
 
-- (id)initWithSlot:(const boost::function<bool ()>*)slot;
+- (id)initWithSlot:(boost::function<bool ()>)slot;
 - (void)fire:(NSTimer*)timer;
 
 @end
 
 @implementation MFTimerHandler
 
-- (id)initWithSlot:(const boost::function<bool ()>*)slot
+- (id)initWithSlot:(boost::function<bool ()>)slot
 {
   self = [super init];
   if (self)
   {
-    callback = new boost::function<bool ()>(*slot);
+    callback = new boost::function<bool ()>(slot);
   }
   return self;
 }
@@ -144,7 +158,7 @@ static std::string get_special_folder(mforms::FolderType type)
 
 static void util_add_timeout(float interval, const boost::function<bool ()> &callback)
 { 
-  MFTimerHandler *handler = [[MFTimerHandler alloc] initWithSlot: &callback];
+  MFTimerHandler *handler = [[MFTimerHandler alloc] initWithSlot:callback];
   
   [NSTimer scheduledTimerWithTimeInterval:interval
                                    target:handler selector:@selector(fire:)
@@ -300,7 +314,7 @@ static bool util_move_to_trash(const std::string &path)
 @end
 
 
-static void *util_perform_from_main_thread(const boost::function<void* ()> &slot)
+static void *util_perform_from_main_thread(const boost::function<void* ()> &slot, bool wait_response)
 {
   if ([NSThread isMainThread])
     return slot ? slot() : NULL;
@@ -308,9 +322,10 @@ static void *util_perform_from_main_thread(const boost::function<void* ()> &slot
   {
     MainThreadRunner *tmp = [[MainThreadRunner alloc] init];
     tmp->slot = slot;
+    tmp->result = NULL;
     [tmp performSelectorOnMainThread: @selector(perform) 
                           withObject: nil
-                       waitUntilDone: YES];
+                       waitUntilDone: wait_response];
     void *result = tmp->result;
     [tmp release];
     return result;
