@@ -19,10 +19,6 @@
 
 #include "stdafx.h"
 
-#include "mforms/menubar.h"
-#include "mforms/mforms.h"
-#include <algorithm>
-
 #ifndef __APPLE__
   #include <algorithm>
 #endif
@@ -62,13 +58,14 @@ void MenuBase::insert_item(int index, MenuItem *item)
 
 void MenuBase::remove_all()
 {
-  _impl->remove_item(this, 0);
+  _impl->remove_item(this, NULL); // null means remove all
   std::vector<MenuItem*>::iterator iter;
   for (iter = _items.begin(); iter != _items.end(); ++iter)
   {
     (*iter)->_parent = 0;
     (*iter)->release();
   }
+  _items.clear();
 }
 
 void MenuBase::remove_item(MenuItem *item)
@@ -97,7 +94,7 @@ MenuItem *MenuBase::find_item(const std::string &name)
 {
   for (std::vector<MenuItem*>::const_iterator iter = _items.begin(); iter != _items.end(); ++iter)
   {
-    if ((*iter)->name() == name)
+    if ((*iter)->get_name() == name)
       return *iter;
     MenuItem *item;
     if ((item = (*iter)->find_item(name)))
@@ -114,6 +111,21 @@ void MenuBase::validate()
 }
 
 
+MenuBar *MenuBase::get_menubar()
+{
+  if (dynamic_cast<MenuBar*>(this) != NULL)
+    return dynamic_cast<MenuBar*>(this);
+
+  MenuBase *p = _parent;
+  while (p && p->get_parent())
+    p = p->get_parent();
+  
+  if (p)
+    return dynamic_cast<MenuBar*>(p);
+  return 0;
+}
+
+
 MenuItem::MenuItem(const std::string &title, const MenuItemType type)
 : MenuBase()
 {
@@ -171,18 +183,6 @@ void MenuItem::set_name(const std::string &name)
 }
 
 
-MenuBar *MenuItem::get_menubar()
-{
-  MenuBase *p = _parent;
-  while (p && p->get_parent())
-    p = p->get_parent();
-  
-  if (p)
-    return dynamic_cast<MenuBar*>(p);
-  return 0;
-}
-
-
 MenuBar::MenuBar()
 : MenuBase()
 {
