@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -51,6 +51,8 @@ namespace mforms {
     MouseButtonLeft = 0,
     MouseButtonRight = 1,
     MouseButtonMiddle = 2,
+    
+    MouseButtonNone = 0x80, // Sometimes the values for the mouse buttons are hardcoded, so better use a high value for this.
   };
 
 #ifndef DOXYGEN_SHOULD_SKIP_THIS
@@ -64,12 +66,12 @@ namespace mforms {
     int (__stdcall *get_preferred_width)(View *self);
     int (__stdcall *get_preferred_height)(View *self);
     void (__stdcall *set_size)(View *self, int, int);
-    void (__stdcall *set_padding)(View *self, const MySQL::Geometry::Padding&);
+    void (__stdcall *set_padding)(View *self, int, int, int, int); // left, top, right, bottom
 
     int (__stdcall *get_x)(View *self);
     int (__stdcall *get_y)(View *self);
     void (__stdcall *set_position)(View *self, int, int);
-    void (__stdcall *client_to_screen)(View *self, MySQL::Geometry::Point&);
+    void (__stdcall *client_to_screen)(View *self, int&, int&);
 
     void (__stdcall *show)(View *self, bool);
     bool (__stdcall *is_shown)(View *self);
@@ -101,6 +103,7 @@ namespace mforms {
   {
     friend class ControlFactory;
 
+  protected:
     ViewImplPtrs *_view_impl;
   private:
     std::string _name;
@@ -148,9 +151,8 @@ namespace mforms {
     virtual int get_y();
     virtual void set_position(int x, int y);
     virtual void set_size(int width, int height);
-    virtual void set_padding(const MySQL::Geometry::Padding& padding);
-    void client_to_screen(MySQL::Geometry::Point& position);
 
+    void client_to_screen(int& x, int& y);
 
     void show(bool flag= true);
     bool is_shown();
