@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2008, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2008, 2011, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -19,9 +19,6 @@
 
 #include "stdafx.h"
 
-#include "mforms/progressbar.h"
-#include "mforms/mforms.h"
-
 using namespace mforms;
 
 ProgressBar::ProgressBar()
