@@ -26,6 +26,7 @@
 #include "grtsqlparser/module_utils.h"
 #include "base/string_utilities.h"
 #include <sstream>
+#include <boost/lambda/lambda.hpp>
 
 
 using namespace grt;
@@ -42,8 +43,10 @@ Mysql_invalid_sql_parser::Null_state_keeper::~Null_state_keeper()
   _sql_parser->_active_obj_list2= grt::ListRef<db_DatabaseDdlObject>();
   _sql_parser->_active_obj= db_DatabaseDdlObjectRef();
   _sql_parser->_active_grand_obj= db_DatabaseObjectRef();
-  _sql_parser->_create_stub_object.disconnect();
-  _sql_parser->_remove_stub_object.disconnect();
+  static const bool fval = false;
+  boost::function<bool ()> f = boost::lambda::var(fval);
+  _sql_parser->_create_stub_object = boost::bind(f);
+  _sql_parser->_remove_stub_object = boost::bind(f);
 }
 #define NULL_STATE_KEEPER Null_state_keeper _nsk(this);
 
@@ -73,9 +76,9 @@ int Mysql_invalid_sql_parser::parse_triggers(db_TableRef table, const std::strin
   _active_grand_obj= table;
   _active_obj_list= ListRef<db_DatabaseDdlObject>::cast_from(table->triggers());
   _stub_name= "SYNTAX_ERROR_";
-  _process_specific_create_statement= sigc::mem_fun(this, &Mysql_invalid_sql_parser::process_create_trigger_statement);
-  _create_stub_object= sigc::mem_fun(this, &Mysql_invalid_sql_parser::create_stub_trigger);
-  _shape_trigger= sigc::mem_fun(this, &Mysql_invalid_sql_parser::shape_trigger);
+  _process_specific_create_statement= boost::bind(&Mysql_invalid_sql_parser::process_create_trigger_statement, this, _1);
+  _create_stub_object= boost::bind(&Mysql_invalid_sql_parser::create_stub_trigger, this, _1);
+  _shape_trigger= boost::bind(&Mysql_invalid_sql_parser::shape_trigger, this, _1);
 
   _triggers_owner_table= db_mysql_TableRef::cast_from(table);
 
@@ -92,10 +95,10 @@ int Mysql_invalid_sql_parser::parse_routines(db_RoutineGroupRef routine_group, c
     db_mysql_SchemaRef::cast_from(_active_grand_obj->owner())->routines());
   _active_obj_list2= ListRef<db_DatabaseDdlObject>::cast_from(routine_group->routines());
   _stub_name= (*routine_group->name()).append("_SYNTAX_ERROR_");
-  _process_specific_create_statement= sigc::mem_fun(this, &Mysql_invalid_sql_parser::process_create_routine_statement);
-  _create_stub_object= sigc::mem_fun(this, &Mysql_invalid_sql_parser::create_stub_group_routine);
-  _remove_stub_object= sigc::mem_fun(this, &Mysql_invalid_sql_parser::remove_stub_group_routine);
-  _shape_routine= sigc::mem_fun(this, &Mysql_invalid_sql_parser::shape_group_routine);
+  _process_specific_create_statement= boost::bind(&Mysql_invalid_sql_parser::process_create_routine_statement, this, _1);
+  _create_stub_object= boost::bind(&Mysql_invalid_sql_parser::create_stub_group_routine, this, _1);
+  _remove_stub_object= boost::bind(&Mysql_invalid_sql_parser::remove_stub_group_routine, this, _1);
+  _shape_routine= boost::bind(&Mysql_invalid_sql_parser::shape_group_routine, this, _1);
   _case_sensitive_identifiers= false; //! identifiers within a mysql routine body are always case-insensitive?
 
   return parse_invalid_sql_script(sql);
@@ -111,8 +114,8 @@ int Mysql_invalid_sql_parser::parse_routine(db_RoutineRef routine, const std::st
   _active_obj_list= ListRef<db_DatabaseDdlObject>::cast_from(
     db_mysql_SchemaRef::cast_from(_active_obj->owner())->routines());
   _stub_name= "SYNTAX_ERROR_";
-  _process_specific_create_statement= sigc::mem_fun(this, &Mysql_invalid_sql_parser::process_create_routine_statement);
-  _create_stub_object= sigc::mem_fun(this, &Mysql_invalid_sql_parser::create_stub_routine);
+  _process_specific_create_statement= boost::bind(&Mysql_invalid_sql_parser::process_create_routine_statement, this, _1);
+  _create_stub_object= boost::bind(&Mysql_invalid_sql_parser::create_stub_routine, this, _1);
   _case_sensitive_identifiers= false; //! identifiers within a mysql routine body are always case-insensitive?
 
   return parse_invalid_sql_script(sql);
@@ -128,8 +131,8 @@ int Mysql_invalid_sql_parser::parse_view(db_ViewRef view, const std::string &sql
   _active_obj_list= ListRef<db_DatabaseDdlObject>::cast_from(
     db_mysql_SchemaRef::cast_from(_active_obj->owner())->views());
   _stub_name= "SYNTAX_ERROR_";
-  _process_specific_create_statement= sigc::mem_fun(this, &Mysql_invalid_sql_parser::process_create_view_statement);
-  _create_stub_object= sigc::mem_fun(this, &Mysql_invalid_sql_parser::create_stub_view);
+  _process_specific_create_statement= boost::bind(&Mysql_invalid_sql_parser::process_create_view_statement, this, _1);
+  _create_stub_object= boost::bind(&Mysql_invalid_sql_parser::create_stub_view, this, _1);
 
   _sql_script_preamble= "DELIMITER " + _non_std_sql_delimiter + EOL;
   std::string sql_= _sql_script_preamble + sql;
@@ -168,7 +171,7 @@ int Mysql_invalid_sql_parser::parse_invalid_sql_script(const std::string &sql)
 
   build_datatype_cache();
 
-  _process_sql_statement= sigc::mem_fun(this, &Mysql_invalid_sql_parser::process_sql_statement);
+  _process_sql_statement= boost::bind(&Mysql_invalid_sql_parser::process_sql_statement, this, _1);
 
   Mysql_sql_parser_fe sql_parser_fe(_grtm->get_grt());
   sql_parser_fe.ignore_dml= false;
