@@ -53,7 +53,7 @@ public:
   int check_routine(const std::string &sql);
 
 protected:
-  typedef sigc::slot<Parse_result, const SqlAstNode *> Check_sql_statement;
+  typedef boost::function<Parse_result (const SqlAstNode *)> Check_sql_statement;
   Check_sql_statement _check_sql_statement;
 
   int process_sql_statement(const SqlAstNode *tree, ObjectType object_type);
