@@ -1,4 +1,23 @@
-#include "stdafx.h"
+/* 
+ * Copyright (c) 2011, Oracle and/or its affiliates. All rights reserved.
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; version 2 of the
+ * License.
+ * 
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+ * GNU General Public License for more details.
+ * 
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
+ * 02110-1301  USA
+ */
+
+#include "tut_stdafx.h"
 
 #ifndef _WIN32
 #include <sstream>
@@ -127,6 +146,7 @@ void Test_object_base<mysql_sql_statement_decomposer>::test_sql(const std::strin
 {
   /* parse sql */
   SelectStatement::Ref select_statement(new SelectStatement());
+  ensure("Decomposer load error",sql_statement_decomposer.get() != NULL);
   int res= sql_statement_decomposer->decompose_query(sql, select_statement);
   ensure("Failed to parse statement: " + sql, (res == 1));
   
