@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -186,7 +186,7 @@ static std::string generate_create(db_mysql_ColumnRef column)
 
   sql.append(" ");
 
-  if (column->simpleType().is_valid())
+  if (column->simpleType().is_valid() && column->simpleType()->group().is_valid())
     if (*column->simpleType()->group()->name() == "string" || *column->simpleType()->group()->name() == "text"
       || *column->simpleType()->name() == "ENUM")
     {
@@ -208,7 +208,7 @@ static std::string generate_create(db_mysql_ColumnRef column)
     size_t flags_count= flags.count();
     for(size_t j= 0; j < flags_count; j++)
       sql.append(flags.get(j).c_str()).append(" ");
-  }else if (column->userType().is_valid())
+  }else if (column->userType().is_valid() && !column->userType()->flags().empty())
     sql.append(column->userType()->flags()).append(" ");
 
   if(column->isNotNull())
@@ -423,6 +423,7 @@ class ActionGenerateSQL : public DiffSQLGeneratorBEActionInterface
 {
   TextPadding padding;
 
+  bool _case_sensitive;
   std::string sql;
   std::string comma;
   std::string table_q_name;
@@ -449,7 +450,7 @@ class ActionGenerateSQL : public DiffSQLGeneratorBEActionInterface
   void alter_table_property(std::string& to, const std::string& name, const std::string& value);
 
 public:
-  ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list,grt::GRT *grt);
+  ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list,grt::GRT *grt, const bool case_sensitive);
   virtual ~ActionGenerateSQL();
 
   // create table
@@ -574,8 +575,8 @@ public:
   inline std::string get_name(GrtNamedObjectRef object) const{return ::get_name(object,_use_short_names);};
 };
 
-ActionGenerateSQL::ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list,grt::GRT *grt)
-  : padding(2)
+ActionGenerateSQL::ActionGenerateSQL(grt::ValueRef target, grt::ListRef<GrtNamedObject> obj_list,grt::GRT *grt, const bool case_sensitive)
+  : padding(2), _case_sensitive(case_sensitive)
 {
   SqlFacade::Ref sql_facade = SqlFacade::instance_for_rdbms_name(grt, "Mysql");
   Sql_specifics::Ref sql_specifics = sql_facade ->sqlSpecifics();
@@ -669,6 +670,11 @@ void ActionGenerateSQL::create_table_fks_begin(db_mysql_TableRef)
 
 void ActionGenerateSQL::create_table_fk(db_mysql_ForeignKeyRef fk)
 {
+  grt::StringRef ename = db_mysql_TableRef::cast_from(fk->owner())->tableEngine();
+  db_mysql_StorageEngineRef engine = bec::TableHelper::get_engine_by_name(fk.get_grt(), ename);
+  if (engine.is_valid() && !engine->supportsForeignKeys())
+      return;
+
   sql.append(",\n");
   padding.pad(sql).append(generate_create(fk, padding, _use_short_names));
 
@@ -752,12 +758,13 @@ void ActionGenerateSQL::create_table_max_rows(grt::StringRef value)
 
 void ActionGenerateSQL::create_table_comment(grt::StringRef value)
 {
-  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(value)).append("'");
-  if (!bec::TableHelper::get_document_comment(value).empty())
+  std::string str = value;
+  std::string comment = escape_sql_string(bec::TableHelper::get_sync_comment(str)).append("'");
+  if (!bec::TableHelper::get_document_comment(str).empty())
   {
     comment.append(" /* comment truncated */");  
   };
-  alter_table_property(sql, "\nCOMMENT = '", comment);
+  sql.append("\nCOMMENT = '").append(comment);
 }
 
 void ActionGenerateSQL::create_table_data_dir(grt::StringRef value)
@@ -898,12 +905,12 @@ void ActionGenerateSQL::alter_table_delay_key_write(db_mysql_TableRef table, grt
 
 void ActionGenerateSQL::alter_table_charset(db_mysql_TableRef table, grt::StringRef str)
 {
-  alter_table_property(sql, "CHARACTER SET = ", str.c_str());
+  alter_table_property(sql, "CHARACTER SET = ", str.empty()?"DEFAULT":str.c_str());
 }
 
 void ActionGenerateSQL::alter_table_collate(db_mysql_TableRef table, grt::StringRef str)
 {
-  alter_table_property(sql, "COLLATE = ", str.c_str());
+  alter_table_property(sql, "COLLATE = ", str.empty()?"DEFAULT":str.c_str());
 }
 
 void ActionGenerateSQL::alter_table_comment(db_mysql_TableRef table, grt::StringRef str)
@@ -1264,6 +1271,10 @@ void ActionGenerateSQL::alter_table_fks_begin(db_mysql_TableRef)
 
 void ActionGenerateSQL::alter_table_add_fk(db_mysql_ForeignKeyRef fk)
 {
+  grt::StringRef ename = db_mysql_TableRef::cast_from(fk->owner())->tableEngine();
+  db_mysql_StorageEngineRef engine = bec::TableHelper::get_engine_by_name(fk.get_grt(), ename);
+  if (engine.is_valid() && !engine->supportsForeignKeys())
+      return;
   if(first_fk_create)
     first_fk_create= false;
   else
@@ -1280,6 +1291,11 @@ void ActionGenerateSQL::alter_table_add_fk(db_mysql_ForeignKeyRef fk)
 
 void ActionGenerateSQL::alter_table_drop_fk(db_mysql_ForeignKeyRef fk)
 {
+  grt::StringRef ename = db_mysql_TableRef::cast_from(fk->owner())->tableEngine();
+  db_mysql_StorageEngineRef engine = bec::TableHelper::get_engine_by_name(fk.get_grt(), ename);
+  if (engine.is_valid() && !engine->supportsForeignKeys())
+      return;
+
   if(first_fk_drop)
     first_fk_drop= false;
   else
@@ -1497,7 +1513,7 @@ void ActionGenerateSQL::remember(const GrtNamedObjectRef &obj, const std::string
   }
   else
   {
-    target_map.set(get_full_object_name_for_key(obj), grt::StringRef(sql));
+    target_map.set(get_full_object_name_for_key(obj,_case_sensitive), grt::StringRef(sql));
   }
 }
 
@@ -1513,7 +1529,7 @@ void ActionGenerateSQL::remember_alter(const GrtNamedObjectRef &obj, const std::
     return;
   }
 
-  std::string key= get_full_object_name_for_key(obj);
+  std::string key= get_full_object_name_for_key(obj,_case_sensitive);
 
   if(target_map.has_key(key))
   {
@@ -1547,6 +1563,7 @@ int DbMySQLImpl::generateSQL(GrtNamedObjectRef org_object,
                                         const std::string &address1)
 {
   DiffChange *p= 0;
+  bool case_sensitive = false;
 
   // yes, this is a hack. yes, this should be fixed
   sscanf(address1.c_str(), "%p", &p);
@@ -1560,15 +1577,18 @@ int DbMySQLImpl::generateSQL(GrtNamedObjectRef org_object,
   if(options.has_key("OutputObjectContainer"))
     obj_list= grt::ListRef<GrtNamedObject>::cast_from(options.get("OutputObjectContainer"));
   
+  if(options.has_key("CaseSensitive"))
+    case_sensitive = IntegerRef::cast_from(options.get("OutputObjectContainer")) != 0;
+
   if(result.type() == DictType)
   {
-    ActionGenerateSQL generator = ActionGenerateSQL(result, obj_list, get_grt());
+    ActionGenerateSQL generator = ActionGenerateSQL(result, obj_list, get_grt(),case_sensitive);
     DiffSQLGeneratorBE(options, &generator)
       .process_diff_change(org_object, p, grt::DictRef::cast_from(result));
   }
   else if(result.type() == ListType)
   {
-    ActionGenerateSQL generator = ActionGenerateSQL(result, obj_list, get_grt());
+    ActionGenerateSQL generator = ActionGenerateSQL(result, obj_list, get_grt(),case_sensitive);
     DiffSQLGeneratorBE(options, &generator)
       .process_diff_change(org_object, p, grt::StringListRef::cast_from(result), obj_list);
   }
@@ -1601,15 +1621,15 @@ grt::StringRef DbMySQLImpl::generateReport(GrtNamedObjectRef org_object,
   return retval;
 }
 
-static bool exists_in_map(GrtNamedObjectRef object, DictRef dict)
+static bool exists_in_map(GrtNamedObjectRef object, DictRef dict, const bool case_sensitive = false)
 {
-  std::string qname(get_full_object_name_for_key(object));
+  std::string qname(get_full_object_name_for_key(object, case_sensitive));
   return dict.has_key(qname);
 }
 
-static std::string string_from_map(GrtNamedObjectRef object, DictRef dict)
+static std::string string_from_map(GrtNamedObjectRef object, DictRef dict, const bool case_sensitive = false)
 {
-  std::string qname(get_full_object_name_for_key(object));
+  std::string qname(get_full_object_name_for_key(object, case_sensitive));
   StringRef res= dict.get_string(qname);
 
   //if (!res.is_valid())
@@ -1885,7 +1905,7 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
   };
 
   if (grt)
-    grt->send_output("Generating Script...\n");
+    grt->send_output("Generating Script\n");
 
   gen_create_index= IntOption::get(options, "GenerateCreateIndex") != 0;
   use_short_names= IntOption::get(options, "UseShortNames") != 0;
@@ -1912,7 +1932,7 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
     if (schema->modelOnly())
       continue;
 
-    if ((!use_short_names || gen_use) && (create_map.has_key(get_old_object_name_for_key(schema))))
+    if ((!use_short_names || gen_use) && (create_map.has_key(get_old_object_name_for_key(schema, false))))
     {
       if (gen_schema_drops)
         out_sql.append("DROP SCHEMA IF EXISTS `").append(schema->name().c_str()).append("` ;\n");
@@ -1935,7 +1955,7 @@ static std::string get_export_sql(db_mysql_CatalogRef cat,
     if (grt) 
       grt->send_output(std::string("Processing Schema ").append(schema->name()).append("\n"));
 
-    if ((!use_short_names || gen_use) && (create_map.has_key(get_old_object_name_for_key(schema))))
+    if ((!use_short_names || gen_use) && (create_map.has_key(get_old_object_name_for_key(schema, false))))
       out_sql.append("USE `").append(schema->name().c_str()).append("` ;\n");
 
 
@@ -2267,12 +2287,14 @@ int DbMySQLImpl::makeSQLSyncScript(const grt::DictRef& options,
   return 0;
 }
 
-std::string DbMySQLImpl::makeAlterScript(GrtNamedObjectRef source, GrtNamedObjectRef target)
+std::string DbMySQLImpl::makeAlterScript(GrtNamedObjectRef source, GrtNamedObjectRef target, const int case_sensitive)
 {
   grt::DbObjectMatchAlterOmf omf;
   omf.dontdiff_mask = 3;
+  grt::NormalizedComparer normalizer(get_grt(), case_sensitive != 0);
+  normalizer.init_omf(&omf);
 
-  std::auto_ptr<DiffChange> diff(diff_make(source, target, &omf, grt::NormalizedComparer(get_grt())));
+  boost::shared_ptr<DiffChange> diff = diff_make(source, target, &omf);
   if (!diff.get())
       return "";
 
@@ -2301,7 +2323,7 @@ std::string DbMySQLImpl::makeAlterScript(GrtNamedObjectRef source, GrtNamedObjec
   return script;
 }
 
-std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtNamedObjectRef target, GrtNamedObjectRef obj)
+std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtNamedObjectRef target, GrtNamedObjectRef obj, const int case_sensitive)
 {
   grt::DbObjectMatchAlterOmf omf;
   omf.dontdiff_mask = 5;
@@ -2310,7 +2332,9 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
   DictRef result(get_grt());
 
   options.set("UseFilteredLists", IntegerRef(0));
-  DiffChange* diff= diff_make(source, target, &omf, grt::NormalizedComparer(get_grt()));
+  grt::NormalizedComparer normalizer(get_grt(), case_sensitive != 0);
+  normalizer.init_omf(&omf);
+  boost::shared_ptr<DiffChange> diff = diff_make(source, target, &omf);
 
   std::string sql;
   std::string non_std_sql_delimiter("$$");
@@ -2322,13 +2346,12 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
   }
 
 
-  if (diff)
+  if (diff.get())
   {
-    ActionGenerateSQL generator = ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt());
+    ActionGenerateSQL generator = ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt(),omf.case_sensitive);
     generator.set_put_if_exists(false);
-    DiffSQLGeneratorBE(options, &generator).process_diff_change(source, diff, result);
-    delete diff;
-    std::string objname = get_old_object_name_for_key(obj);
+    DiffSQLGeneratorBE(options, &generator).process_diff_change(source, diff.get(), result);
+    std::string objname = get_old_object_name_for_key(obj, omf.case_sensitive);
     ValueRef change = result.get(objname, StringRef(""));
     if (StringRef::can_wrap(change))
     {
@@ -2345,7 +2368,7 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
 
     if((obj->name() != obj->oldName()) && !obj->oldName().empty())
     {
-      std::string objname = get_full_object_name_for_key(obj);
+      std::string objname = get_full_object_name_for_key(obj,case_sensitive != 0);
       ValueRef new_change = result.get(objname, StringRef(""));
       if (StringRef::can_wrap(new_change))
       {
@@ -2390,10 +2413,10 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
                     }
                     if (!trigger_found)
                     {
-                      std::string trigger_code = result.get_string(get_full_object_name_for_key(db_triggers.get(i)), "");
+                      std::string trigger_code = result.get_string(get_full_object_name_for_key(db_triggers.get(i),case_sensitive != 0), "");
                       if (!trigger_code.empty())
                       {
-                        sql.append("USE ").append(db_table->owner()->name()).append(";\n").append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
+                        sql.append("USE `").append(db_table->owner()->name()).append("`;\n").append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
                         sql.append(trigger_code).append(non_std_sql_delimiter).append("\nDELIMITER ;\n");
                       }
                     }
@@ -2406,10 +2429,10 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
       }
       for(size_t c= triggers.count(), i= 0; i < c; i++)
       {
-        std::string trigger_code = result.get_string(get_full_object_name_for_key(triggers.get(i)), "");
+        std::string trigger_code = result.get_string(get_full_object_name_for_key(triggers.get(i),case_sensitive != 0), "");
         if (!trigger_code.empty())
         {
-          sql.append("USE ").append(table->owner()->name()).append(";\n").append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
+          sql.append("USE `").append(table->owner()->name()).append("`;\n").append("\nDELIMITER ").append(non_std_sql_delimiter).append("\n\n");
           sql.append("DROP TRIGGER IF EXISTS ").append(table->owner()->name()).append(".");
           sql.append(triggers.get(i)->name()).append(non_std_sql_delimiter).append("\n");
           sql.append(trigger_code).append(non_std_sql_delimiter).append("\nDELIMITER ;\n");
@@ -2438,16 +2461,15 @@ std::string DbMySQLImpl::makeAlterScriptForObject(GrtNamedObjectRef source, GrtN
 // This function is used from scripts and HTML report generator.
 std::string DbMySQLImpl::makeCreateScriptForObject(GrtNamedObjectRef object)
 {
-  grt::DbObjectMatchRecreateOmf omf;
-  omf.dontdiff_mask = 3;
-
   DictRef options(get_grt());
   DictRef result(get_grt());
 
   ValueRef parent;
 
-  StringListRef list(get_grt());
-  list.insert(get_old_object_name_for_key(object));
+  //TODO: check how this list is expected to be used
+  //there should be a way to generate one table SQL only not sql for whole doc
+//  StringListRef list(get_grt());
+//  list.insert(get_old_object_name_for_key(object));
 
   if (object.is_instance(db_Schema::static_class_name()))
     parent= object->owner();
@@ -2465,16 +2487,18 @@ std::string DbMySQLImpl::makeCreateScriptForObject(GrtNamedObjectRef object)
     return "";
 
   options.set("UseFilteredLists", IntegerRef(0));
-  DiffChange* diff= diff_make(ValueRef(), parent, NULL, grt::NormalizedComparer(get_grt()));
+  default_omf omf;
+  grt::NormalizedComparer normalizer(get_grt());
+  normalizer.init_omf(&omf);
+  boost::shared_ptr<DiffChange> diff = diff_make(ValueRef(), parent, &omf);
 
   std::string sql;
 
-  if (diff)
+  if (diff.get())
   {
-    ActionGenerateSQL generator = ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt());
-    DiffSQLGeneratorBE(options, &generator).process_diff_change(ValueRef(), diff, result);
-    delete diff;
-    sql= result.get_string(get_full_object_name_for_key(object), "");
+    ActionGenerateSQL generator = ActionGenerateSQL(result, grt::ListRef<GrtNamedObject>(),get_grt(),omf.case_sensitive);
+    DiffSQLGeneratorBE(options, &generator).process_diff_change(ValueRef(), diff.get(), result);
+    sql= result.get_string(get_full_object_name_for_key(object, omf.case_sensitive), "");
   }
 
   return sql;
@@ -2488,6 +2512,50 @@ grt::ListRef<db_mysql_StorageEngine> DbMySQLImpl::getKnownEngines()
   return _known_engines;
 }
 
+// checks whether the 2nd version number is the same or newer than the 1st one
+inline bool match_version(int ref_major, int ref_minor, int ref_revision,
+                          int major, int minor, int revision)
+{
+  if (major > ref_major)
+    return true;
+  if (minor < ref_minor)
+    return false;
+  if (minor > ref_minor)
+    return true;
+  if (minor < ref_minor)
+    return false;
+  if (revision >= ref_revision)
+    return true;
+  return false;
+}
+
+
+grt::DictRef DbMySQLImpl::getTraitsFromServerVariables(const grt::DictRef &variables)
+{
+  grt::DictRef traits(get_grt());
+  int major = 0, minor = 0, revision = 0;
+  
+  std::string version;
+  if (variables.has_key("version"))
+    version = variables.get_string("version");
+  
+  sscanf(version.c_str(), "%i.%i.%i", &major, &minor, &revision);
+  
+  if (match_version(5, 5, 3,  major, minor, revision))
+  {
+    traits["maxTableCommentLength"] = grt::IntegerRef(2048);
+    traits["maxIndexCommentLength"] = grt::IntegerRef(1024);
+    traits["maxColumnCommentLength"] = grt::IntegerRef(1024);
+  }
+  else
+  {
+    traits["maxTableCommentLength"] = grt::IntegerRef(60);
+    traits["maxIndexCommentLength"] = grt::IntegerRef(0);
+    traits["maxColumnCommentLength"] = grt::IntegerRef(255);
+  }
+  
+  return traits;
+}
 
 grt::ListRef<db_UserDatatype> DbMySQLImpl::getDefaultUserDatatypes(db_mgmt_RdbmsRef rdbms)
 {
