@@ -1,5 +1,5 @@
 /* 
- * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2009, 2012, Oracle and/or its affiliates. All rights reserved.
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -105,6 +105,7 @@ static void call_refresh(DbMysqlSchemaEditor *self)
 
 - (void) dealloc
 {
+  [topView release];
   delete mBackEnd;
   [super dealloc];
 }
@@ -149,7 +150,13 @@ static void call_refresh(DbMysqlSchemaEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return topView;
+}
+
+
+- (void)pluginDidShow: (id)sender
+{
+  [[tabView window] makeFirstResponder: nameText];
 }
 
 
@@ -187,4 +194,8 @@ static void call_refresh(DbMysqlSchemaEditor *self)
   return mBackEnd;
 }
 
+- (NSImage*)titleIcon
+{
+  return [NSImage imageNamed: @"tab_icon_db.Schema.png"];
+}
 @end
