@@ -257,7 +257,7 @@ static void call_refresh(DbMysqlRoleEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return [tabView superview];
 }
 
 
