@@ -128,7 +128,7 @@ static void call_refresh(DbMysqlUserEditor *self)
 - (NSView*)dockableView
 {
   // the view to be docked to the main window
-  return tabView;
+  return [tabView superview];
 }
 
 
