@@ -205,7 +205,7 @@ private:
   DiffTreeBE(const std::vector<std::string>& schemata,
              db_mysql_CatalogRef model_catalogRef, 
              db_mysql_CatalogRef external_catalog, 
-             DiffChange* diffchange, DiffNodeController controller = DiffNodeController());
+             boost::shared_ptr<DiffChange> diffchange, DiffNodeController controller = DiffNodeController());
 
   virtual int count_children(const bec::NodeId &);
   virtual bec::NodeId get_child(const bec::NodeId &, int);
