@@ -18,7 +18,7 @@
 import grt
 from grt import modules
 from db_utils import QueryError, escape_sql_string
-from wb_common import dprint_ex, PermissionDeniedError
+from wb_common import PermissionDeniedError
 from grt import modules;
 
 import hashlib
@@ -161,7 +161,7 @@ SecurityAdminRoles = [
     ["Create_priv", "Drop_priv", "Grant_priv", "Event_priv", "Alter_priv", "Delete_priv", 
     "Index_priv", "Insert_priv", "Select_priv", "Update_priv", "Create_tmp_table_priv", "Lock_tables_priv",
     "Trigger_priv", "Create_view_priv", "Show_view_priv", "Create_routine_priv", "Alter_routine_priv", "Show_db_priv"]),
-("DBDesigner", "rights to create and reverse engineer any dabase schema", 
+("DBDesigner", "rights to create and reverse engineer any database schema", 
     ["Create_priv", "Alter_priv", "Index_priv", "Trigger_priv", "Create_view_priv",
     "Show_view_priv", "Create_routine_priv", "Alter_routine_priv", "Show_db_priv", "Routine_manage_attr"]),#, "IS_read_all_attr"
 ("ReplicationAdmin", "rights needed to setup and manage replication", 
@@ -197,7 +197,7 @@ def acl_compare(t1, t2):
     return get_acl_sort(t1) - get_acl_sort(t2)
 
 
-class AdminSecurity:
+class AdminSecurity(object):
     def __init__(self, ctrl_be):
         self.ctrl_be = ctrl_be
         self._accounts = []
@@ -338,15 +338,22 @@ class AdminSecurity:
     def copy_account(self, account):
         copy = account.copy()
         copy.is_commited = False
+        copy.username += '_copy'
         self._account_info_cache[copy.username+"@"+copy.host] = copy
         self._accounts.append((copy.username, copy.host))
         return copy
 
 
     def create_account(self):
+        def unique_name(user, host, counter=None):
+            name = user + ( str(counter) if counter else '' )
+            if (name, host) in self._accounts:
+                name = unique_name(user, host, counter+1 if isinstance(counter, int) else 1)
+            return name
+            
         acct = AdminAccount(self)
-        acct.username = "newuser"
         acct.host = "%"
+        acct.username = unique_name('newuser', acct.host)
         self._account_info_cache[acct.username+"@"+acct.host] = acct
         self._accounts.append((acct.username, acct.host))
         return acct
@@ -391,7 +398,7 @@ class AdminSecurity:
 
 
 
-class AdminUserDbPrivEntry:
+class AdminUserDbPrivEntry(object):
     host = None
     db = None
     privileges = set()
@@ -409,7 +416,7 @@ class AdminUserDbPrivEntry:
 
 
 
-class AdminUserDbPrivs:
+class AdminUserDbPrivs(object):
     _owner = None
     username = None    
     entries = []
@@ -493,7 +500,7 @@ class AdminUserDbPrivs:
 
 
 
-class AdminAccount:
+class AdminAccount(object):
     _owner = None
     
     _orig_username = None
